<?php

namespace Framework\Controllers;

use Framework\Foundation\Config;
use Framework\Modules\PaayThreeDs;
use Framework\Modules\UserAccess;
use Framework\Foundation\Pipeline;
use Framework\Foundation\Helper;

class CrmController extends BaseController
{

    public function prospect($request)
    {
        return (new Pipeline())->send($request)->through([
                    \Framework\Modules\Crm\Middleware\Shield::class,
                    \Framework\Modules\Crm\Middleware\FilterRequestObject::class,
                    \Framework\Modules\Crm\Middleware\NewProspect\FilterFormData::class,
                ])->then(function($request)
                {
                    $response = $this->crmAdapter->newProspect($request->json);
                    if ($response['status'])
                    {
                        return pre_response_array(null, true, $response['api_response']);
                    }

                    return pre_response_array(!empty($response['ex_message']) ? $response['ex_message'] : '', false, $response['api_response']);
                });
    }

    public function checkout($request)
    {
        return (new Pipeline())->send($request)->through([
                    \Framework\Modules\Crm\Middleware\Checkout\CheckConfigSettings::class,
                    \Framework\Modules\Crm\Middleware\Shield::class,
                    \Framework\Modules\Crm\Middleware\FilterRequestObject::class,
                    \Framework\Modules\Crm\Middleware\IsBlockedCustomer::class,
                    \Framework\Modules\Crm\Middleware\Checkout\FilterFormData::class,
                    \Framework\Modules\Crm\Middleware\MaxMindScoreCheck::class,
                    \Framework\Modules\Crm\Middleware\QualityScore::class,
                ])->then(function($request)
                {
                    $formData           = $request->json;
                    $orderData          = $request->orderData;
                    $insureshipCheckbox = $request->insureshipCheckbox;

                    $response = $this->crmAdapter->newOrderWithProspect($formData);

                    if (!$response['status'])
                    {
                        return array_merge(pre_response_array(null, false, $response['api_response']), ['isActiveCaptcha' => UserAccess::_showGoogleCaptcha(),]);
                    }
                    
                    if($response['isPaypalExpressReponse']){
                        return $response;
                    }

                    if (Config::get('offer_details.split_charge') && $insureshipCheckbox)
                    {
                        $orderData['order_type']    = 'SplitOrder';
                        $_SESSION['split_order_id'] = ($this->apiType == 'response') ? (string) $this->order($orderData, 1) : $this->order($orderData, 1);
                    }

                    $response['api_response']['prospectId'] = ($this->apiType == 'response') ? (string) $formData['prospectId'] : $formData['prospectId'];
                    $_SESSION['checkout_order_id']          = ($this->apiType == 'response') ? (string) $response['api_response']['orderId'] : $response['api_response']['orderId'];

                    if (!empty($response['api_response']['orderId']) && in_array($formData['cc_type'], ['master', 'mastercard']))
                    {
                        $this->captureOrderDetails($response['api_response']['orderId'], $_SESSION['full_order_data_copy']['email']);
                    }

                    $responseMeta = [];
                    if ($this->apiType == 'limelight')
                    {
                        $responseMeta = ['declineReason' => !empty($response['isOrderDecline']) ? $response['api_response'] : null];
                    }

                    if ($this->apiType == 'konnektive')
                    {
                        $responseMeta = ['declineReason' => !empty($response['isOrderDecline']) ? $response['api_response']['response_message'] : null];
                    }

                    return array_merge(pre_response_array(null, true, $response['api_response']), $responseMeta, ['isActiveCaptcha' => UserAccess::_showGoogleCaptcha(),]);
                });
    }

    public function offerWithoutShipping($request)
    {
        return (new Pipeline())->send($request)->through([
            \Framework\Modules\Crm\Middleware\IsBlockedCustomer::class,
            \Framework\Modules\Crm\Middleware\Shield::class,
            \Framework\Modules\Crm\Middleware\FilterRequestObject::class,
            \Framework\Modules\Crm\Middleware\Offer\FilterFormData::class,
            \Framework\Modules\Crm\Middleware\MaxMindScoreCheck::class,
            \Framework\Modules\Crm\Middleware\QualityScore::class,
        ])->then(function($request)
        {

            $formData = $request->json;
            $response = $this->order($formData, 1);

            if ($response['status'] == 'success')
            {
                if(empty($response['isPaypalExpressReponse'])){
                    $_SESSION['checkout_order_id'] = $response['apiResponse']['orderId'];

                    if (Config::get('offer_details.split_charge') && $request->insureshipCheckbox)
                    {
                        $formData['order_type']     = 'SplitOrder';
                        $_SESSION['split_order_id'] = $this->order($formData, 1);
                    }
                }
            }

            return $response;
        });
    }

    public function offer($request)
    {
        return (new Pipeline())->send($request)->through([
                    \Framework\Modules\Crm\Middleware\IsBlockedCustomer::class,
                    \Framework\Modules\Crm\Middleware\Shield::class,
                    \Framework\Modules\Crm\Middleware\FilterRequestObject::class,
                    \Framework\Modules\Crm\Middleware\Offer\FilterFormData::class,
                    \Framework\Modules\Crm\Middleware\MaxMindScoreCheck::class,
                    \Framework\Modules\Crm\Middleware\QualityScore::class,
                ])->then(function($request)
                {
                    $formData = $request->json;
                    $response = $this->order($formData, 1);

                    if ($response['status'] == 'success')
                    {
                        if(empty($response['isPaypalExpressReponse'])){
                            $_SESSION['checkout_order_id'] = $response['apiResponse']['orderId'];

                            if (Config::get('offer_details.split_charge') && $request->insureshipCheckbox)
                            {
                                $formData['order_type']     = 'SplitOrder';
                                $_SESSION['split_order_id'] = $this->order($formData, 1);
                            }
                        }
                    }

                    return $response;
                });
    }

    public function upsell($request)
    {
        return (new Pipeline())->send($request)->through([
                    \Framework\Modules\Crm\Middleware\IsBlockedCustomer::class,
                    \Framework\Modules\Crm\Middleware\Upsell\FilterRequestObject::class,
                    \Framework\Modules\Crm\Middleware\Upsell\FilterFormData::class,
                ])->then(function($request)
                {

                    $upsellStep              = (int) $this->getCurrentStep('upsell');
                    $upsellData              = $request->upsellData;
                    $response                = $this->order($upsellData, $upsellStep);
                    $_SESSION['upsell_step'] = $upsellStep + 1;
                    $response['upsell_step'] = $upsellStep - 1;

                    if ($response['status'] == 'success')
                    {
                        if(empty($response['isPaypalExpressReponse'])){
                            if (Config::get('offer_details.step' . $upsellStep . '.splitChargeDetails') && $request->insureshipCheckbox)
                            {
                                $splitOrderData               = $_SESSION['full_order_data'];
                                $splitOrderData['order_type'] = 'SplitOrder';

                                if (isset($_SESSION['upsell_split_order_ids'][$response['apiResponse']['orderId']]))
                                {
                                    if ($this->apiType == 'response')
                                    {
                                        array_push($_SESSION['upsell_split_order_ids'][$response['apiResponse']['orderId']], (string) $this->order($splitOrderData, $upsellStep));
                                    }
                                    else
                                    {
                                        array_push($_SESSION['upsell_split_order_ids'][$response['apiResponse']['orderId']], $this->order($splitOrderData, $upsellStep));
                                    }
                                }
                                else
                                {
                                    if ($this->apiType == 'response')
                                    {
                                        $_SESSION['upsell_split_order_ids'][(string) $response['apiResponse']['orderId']][] = (string) $this->order($splitOrderData, $upsellStep);
                                    }
                                    else
                                    {
                                        $_SESSION['upsell_split_order_ids'][$response['apiResponse']['orderId']][] = $this->order($splitOrderData, $upsellStep);
                                    }
                                }
                            }
                        }

                    }

                    return $response;
                });
    }

    public function switchUpsell()
    {
        $upsellStep = (int) $this->getCurrentStep('upsell');
        $success    = false;

        try
        {
            if (file_exists('templates/upsell' . $upsellStep . '.html'))
            {
                $_SESSION['upsell_step'] = $upsellStep + 1;
                $success                 = true;
            }
        }
        catch (\Exception $e)
        {
            
        }
        return pre_response_array(null, $success);
    }

    public function validateCoupon($request)
    {
        $formData = $request->json;
        if (isset($_SESSION['prospect_shipping_data']))
        {
            $formData['email'] = $_SESSION['prospect_shipping_data']['email'];
        }

        $response = $this->crmAdapter->validateCoupon($formData);
        return pre_response_array(null, !empty($response['status']) ? true : false, $response['api_response']);
    }

    public function remoteUpsell($request)
    {
        $formData = $request->json;
        $response = $this->crmAdapter->remoteUpsell([
            'previousOrderId' => array_get($formData, 'previousOrderId', ''),
            'campaignId'      => array_get($formData, 'campaignId', Config::get('offer_details.step1.campaign_id')),
            'shippingId'      => Config::get('selected_shipping_profile')->get('shipping_profile_crm_id'),
            'productId'       => array_get($formData, 'productId', Config::get('offer_details.step1.selectedProduct.0.product_id')),
            'product_qty_1'   => array_get($formData, 'productQty', Config::get('offer_details.step1.selectedProduct.0.crm_product_qty')),
        ]);

        return pre_response_array(null, ($response['status'] == 'success') ? true : false, $response['api_response']);
    }

    public function orderCancel($request)
    {
        return (new Pipeline())->send($request)->through([
                    \Framework\Modules\Crm\Middleware\OrderCancel\FilterFormData::class,
                    \Framework\Modules\Crm\Middleware\OrderCancel\OrderView::class,
                ])->then(function($request)
                {
                    $orderCancelresponse = $this->crmAdapter->cancelOrder(
                            $this->crmAdapter->prepareDataForCancel($request->orderViewResponse, $request->json)
                    );

                    if (empty($orderCancelresponse['status']) || ($orderCancelresponse['status'] != 'success'))
                    {
                        return pre_response_array(null, false, ['response_message' => $orderCancelresponse['apiResponse']]);
                    }

                    $mailResponse = $this->sendOrderCancellationEmail($request);

                    return pre_response_array(null, true, ['response_message' => $orderCancelresponse['apiResponse']], compact('mailResponse'));
                });
    }

    public function paay3Ds()
    {
        $response = PaayThreeDs::paayVerify();

        return $response;
    }

    // Limelight CRM specific routes
    public function Ll3DValidation()
    {
        try
        {
            if ($this->apiType != 'limelight')
                return true;

            $total_amount = $_SESSION['ll3dData']['total_amount'];
            $x_card_num   = $_SESSION['ll3dData']['cc_number'];
            $x_exp_month  = $_SESSION['ll3dData']['expmonth'];
            $x_exp_year   = $_SESSION['ll3dData']['expyear'];
            $gateway_id   = Config::get('offer_details.step1.crm_force_gateway_id', 0);
            $x_relay_url  = sprintf('//%sll3dpost', Helper::getBaseUrl());
            $x_amount     = empty($total_amount) ? array_get($_SESSION, 'twig_tokens.total_price', '0.00') : $total_amount;

            $response = $this->crmAdapter->checkout3DValidation(
                    compact('gateway_id', 'x_relay_url', 'x_amount', 'x_card_num', 'x_exp_month', 'x_exp_year')
            );

            if ($response['status'])
            {
                return pre_response_array(null, true, $response['api_response']);
            }

            return pre_response_array(!empty($response['ex_message']) ? $response['ex_message'] : '', false, $response['api_response']);
        }
        catch (\Exception $ex)
        {
            return pre_response_array($ex->getMessage(), false);
        }
    }

    public function Ll3DPost($request)
    {
        try
        {
            $response = [
                'cavv'              => $request->request->get('cavv'),
                'eci'               => $request->request->get('eci'),
                'xid'               => $request->request->get('xid'),
                'verify_3d_temp_id' => $request->request->get('verify_3d_temp_id'),
                'error'             => false,
            ];

            if (in_array($request->request->get('x_card_num'), Config::get('payment_methods.credit_card.test_cards', [])))
            {
                return $response;
            }

            if (empty($response['cavv']) || empty($response['eci']) || empty($response['xid']))
            {
                throw new \Exception('Invalid credit card provided, Please Provide a valid card information.');
            }

            return $response;
        }
        catch (\Exception $ex)
        {
            return pre_response_array($ex->getMessage(), false);
        }
    }

    public function saveLL3dData($request)
    {
        $_SESSION['ll3dData'] = array(
            'total_amount' => array_get($request->json, 'total_amount'),
            'cc_number'    => array_get($request->json, 'cc_number'),
            'expmonth'     => array_get($request->json, 'expmonth'),
            'expyear'      => array_get($request->json, 'expyear'),
        );
    }

    //** Class Private methods starts */
    private function order(Array $data, $step = 0)
    {
        $data['step'] = $step;
        return (new Pipeline())->send($data)->through([
                    \Framework\Modules\Crm\Middleware\Order\ProcessDataForSixCrm::class,
                ])->then(function($data)
                {
                    $order_type = $data['order_type'];
                    unset($data['order_type']);

                    if ($order_type == 'SplitOrder')
                    {
                        $response       = $this->crmAdapter->placeSplitOrder($data, $data['step']);
                        $split_order_id = $response['status'] ? $response['api_response']['orderId'] : '';
                        return $split_order_id;
                    }

                    if ($order_type == 'UpsellOrder')
                    {
                        $response = $this->crmAdapter->placeUpsellOrder($data, $data['step']);

                        if (!$response['status'])
                        {
                            return [
                                'status'          => 'error',
                                'isActiveCaptcha' => UserAccess::_showGoogleCaptcha(),
                                'apiResponse'     => $response['api_response']
                            ];
                        }

                        if (isset($_SESSION['upsell_order_ids']))
                        {
                            array_push($_SESSION['upsell_order_ids'], $response['api_response']['orderId']);
                        }
                        else
                        {
                            $_SESSION['upsell_order_ids'][] = $response['api_response']['orderId'];
                        }

                        return ['status' => 'success', 'apiResponse' => $response['api_response']];
                    }

                    if ($order_type == 'NewOrder')
                    {                
                        $response = $this->crmAdapter->placeNewOrder($data, $data['step']);
                        if ($response['status'])
                        {
                            if($response['isPaypalExpressReponse']){
                                $response['apiResponse'] = $response['api_response'];
                                $response['status'] = 'success';
                                unset($response['api_response']);
                                return $response;
                            }

                            $declineReason = (!empty($response['isOrderDecline'])) ? $response['api_response'] : null;
                            return ['status' => 'success', 'apiResponse' => $response['api_response'], 'declineReason' => $declineReason];
                        }
                        else
                        {
                            return [
                                'status'                 => 'error',
                                'isActiveCaptcha'        => UserAccess::_showGoogleCaptcha(),
                                'isDeclineReasonEnabled' => Config::get('decline_reason'),
                                'apiResponse'            => $response['api_response']
                            ];
                        }
                    }
                });
    }

    private function sendOrderCancellationEmail($request)
    {

        if (!Config::get('addons.smtp.config_id')) return [];

        $response = $this->httpClient->post(Helper::uniApiEndpoint() . 'ajax/send-mail', ['body' => [
                'order_id'          => array_get($request->json, 'orderId'),
                'email'             => array_get($request->json, 'email'),
                'token'             => array_get($request->orderViewResponse, 'placeholderToken'),
                'email_template_id' => array_get($request->json, 'email_template_id', ''),
                'smtp_setting_id'   => Config::get('addons.smtp.config_id', ''),
                'mail_type'         => 65,
                'funnel_id'         => Helper::getFunnelId()
        ]]);

        $apiResponseMessage = 'The request couldn\'t be completed!';

        try
        {
            $apiResponse        = json_decode($response->getBody()->getContents(), true);
            $apiResponseMessage = array_get($apiResponse, 'message', 'The request couldn\'t be completed!');
        }
        catch (\Exception $e) { }

        if ($response->getStatusCode() != 200)
        {
            return pre_response_array(null, false, ['response_message' => $apiResponseMessage]);
        }

        return pre_response_array(null, $apiResponse['status'] == 'success' ? true : false, ['response_message' => $apiResponseMessage]);
    }

    private function captureOrderDetails($order_id, $email)
    {
        if (!Helper::getFunnelId()) return;

        if ((in_array($this->apiType, ['limelight', 'sixcrm', 'konnektive'])) && Config::get('consent_notification'))
        {
            $requestUrL       = Helper::uniApiEndpoint() . 'api/transactions/capture';
            $consentDayBefore = intval(Config::get('consent_day_before'));
            $formPostParams   = [
                'funnel_id'          => Helper::getFunnelId(),
                'order_id'           => $order_id,
                'email'              => $email,
                'consent_day_before' => !empty($consentDayBefore) ? $consentDayBefore : NULL,
            ];
            $response         = $this->httpClient->post($requestUrL, [
                'body'    => json_encode($formPostParams),
                'headers' => [
                    'Content-Type' => 'application/json',
                    'Api-Token'    => Config::get('uni_api_token'),
                ],
            ]);
            $httpResponse     = $response->getStatusCode();
            $apiResponse      = json_decode($response->getBody()->getContents(), true);
        }
    }

    public function clientSecret($request)
    {
        return (new Pipeline())->send($request)->through([
            \Framework\Modules\Crm\Middleware\Checkout\CheckConfigSettings::class,
            \Framework\Modules\Crm\Middleware\Shield::class,
            \Framework\Modules\Crm\Middleware\FilterRequestObject::class,
        ])->then(function($request)
        {
            $paymentIntentId = null;
            $formData = $request->json;
            if(!empty($request->get('paymentIntentId')) && $request->get('paymentIntentId') != ''){
                $paymentIntentId = $request->get('paymentIntentId');
            }
            return $this->crmAdapter->constructEnv()->paymentIntentsSecret($formData, $paymentIntentId);;
        });
    }

}
