<?php

namespace Framework\Controllers;

use Framework\Foundation\Config;
use Framework\Modules\UserAccess;

class StaticPageController extends \Framework\Controllers\BaseController
{

    public function __construct()
    {
        parent::__construct();
    }

    public function indexPage($request)
    {
        @session_destroy();
        @session_start();

        Config::set('current_step', $this->getCurrentStep('index'));
        Config::set('current_page', 'index');

        $token = $this->templateToken($request);
        return $this->templateEngine->loadview('index.html', $token);
    }

    public function checkoutPage($request)
    {
        Config::set('current_step', $this->getCurrentStep('checkout'));
        Config::set('current_page', 'checkout');

        $token = $this->templateToken($request);

        $url = parse_url($request->getUri());

        return $this->templateEngine->loadview('checkout.html', $token);
    }

    public function upsellPage($request)
    {
        $currentUpsellStep = $this->getCurrentStep('upsell') - 1;
        Config::set('current_step', $this->getCurrentStep('upsell'));
        Config::set('current_page', 'upsell');

        $token = $this->templateToken($request);

        return $this->templateEngine->loadview('upsell' . $currentUpsellStep . '.html', $token);
    }

    public function thankyouPage($request)
    {
        if (!empty($_SESSION['full_order_data']))
        {
            unset($_SESSION['full_order_data']);
        }

        Config::set('current_step', $this->getCurrentStep('thankyou'));
        Config::set('current_page', 'thankyou');

        $token = $this->templateToken($request);

        return $this->templateEngine->loadview('thankyou.html', $token);
    }

    /**
     * Load static pages
     * @deprecated deprecated since version 4.0.0, use staticPage() instead
     * @param Mixed $request
     * @return String
     */
    public function openPages($request)
    {
        Config::set('current_step', $this->getCurrentStep());
        Config::set('current_page', 'static');

        $token = $this->templateToken($request);

        return $this->templateEngine->loadview(null, $token);
    }

    /**
     * Load static pages
     * @param Mixed $request
     * @return String
     */
    public function staticPage($request)
    {
        Config::set('current_step', $this->getCurrentStep());
        Config::set('current_page', 'static');

        $token = $this->templateToken($request);

        return $this->templateEngine->loadview(null, $token);
    }

    public function directAccess($request)
    {
        $token = $this->templateToken($request);

        $path = Config::get('render_view');
        if (file_exists($path))
        {
            $mime = mime_content_type($path);
            if (substr($path, -4) == ".css")
            {
                $mime = "text/css";
            }

            header('Content-Type: ' . $mime);
            include $path;
        }
        else
        {
            Config::set('render_view', '404');
            return $this->templateEngine->loadview(null, $token);
        }
    }

    public function testPage($request)
    {
        echo "<pre>";
        $config = Config::get();
        echo "Config:<br>";
        print_r($config);
        echo "<br>Session:<br>";
        print_r($_SESSION);
        die;
    }

    /**
     * Creating Object for calling the image library
     * @param Mixed $request
     */
    public function generateImage($request)
    {
        $imageObject = new \Framework\Modules\ImageCreator();
        header('Content-Type: image/jpeg');

        $imageText = strip_tags($request->get('text'));
        $fontSize  = trim($request->get('fontSize'));
        $fontName  = trim($request->get('fontName'));
        $hexColour = trim($request->get('hexColour'));
        $img       = $imageObject->createImage($imageText, $fontSize, $fontName, $hexColour);
        echo $img;
        exit();
    }

    public function Mbilling3dsResponse($request)
    {
        $query       = parse_url($_SERVER['REQUEST_URI'], PHP_URL_QUERY);
        parse_str($query, $params);
        $token       = $params['token-id'];
        $crmData     = $_SESSION['3ds_meta']['crm_data'];
        $response    = $response    = $this->crmAdapter->process3DOrder($token, $crmData);
        $urlParts    = parse_url($_SESSION['3ds_meta']['crm_data']['current_url']);
        $queryParams = array();
        if (!empty($urlParts['query']))
        {
            parse_str($urlParts['query'], $queryParams);
            unset($queryParams['error']);
        }

        if (empty($response['status']))
        {
            $redirectPath         = empty($urlParts['path']) ? '/' : $urlParts['path'];
            $queryParams['error'] = $response['api_response']['response_message'];
        }
        else
        {
            $redirectPath               = $_SESSION['3ds_meta']['crm_data']['next_url'];
            $queryParams['order_id']    = $response['api_response']['orderId'];
            $queryParams['customer_id'] = $response['api_response']['customerId'];
        }
        header(sprintf(
                        'Location: %s?%s', $redirectPath, http_build_query($queryParams)
        ));
        exit;
    }

    public function limelight3DResponse($request)
    {

        $data = array(
            'cavv' => $request->request->get('cavv'),
            'eci'  => $request->request->get('eci'),
            'xid'  => $request->request->get('xid'),
        );

        $crmData = array_filter(array_replace_recursive(
                        $_SESSION['3ds_meta']['crm_data'], $data
        ));

        $response = $this->crmAdapter->process3DOrder($_SESSION['3ds_meta']['method'], $crmData);

        $urlParts    = parse_url($_SESSION['3ds_meta']['current_url']);
        $queryParams = array();
        if (!empty($urlParts['query']))
        {
            parse_str($urlParts['query'], $queryParams);
            unset($queryParams['error']);
        }

        if (empty($response['status']))
        {
            $redirectPath         = empty($urlParts['path']) ? '/' : $urlParts['path'];
            $queryParams['error'] = $response['api_response']['response_message'];
        }
        else
        {
            $redirectPath               = $_SESSION['3ds_meta']['next_url'];
            $queryParams['order_id']    = $response['api_response']['orderId'];
            $queryParams['customer_id'] = $response['api_response']['customerId'];
        }

        header(sprintf(
                        'Location: %s?%s', $redirectPath, http_build_query($queryParams)
        ));
        exit;
    }

    public function konnektive3dsResponse()
    {
        $urlParts    = parse_url($_SESSION['gateway_meta']['current_url']);
        $queryParams = array();

        if (!empty($urlParts['query']))
        {
            parse_str($urlParts['query'], $queryParams);
            unset($queryParams['error']);
        }

        if (!empty($_REQUEST['orderData']))
        {
            $orderData = json_decode($_REQUEST['orderData'], true);
        }

        if (
                !empty($orderData['orderStatus']) &&
                $orderData['orderStatus'] === 'COMPLETE'
        )
        {
            $redirectPath               = $_SESSION['gateway_meta']['next_url'];
            $queryParams['order_id']    = $orderData['orderId'];
            $queryParams['customer_id'] = $orderData['customerId'];

            $_SESSION['__orders__'][$orderData['orderId']] = $orderData['items'];
        }
        else
        {
            $redirectPath         = empty($urlParts['path']) ? '/' : $urlParts['path'];
            $queryParams['error'] = $_REQUEST['errorMsg'];
        }

        header(sprintf(
                        'Location: %s?%s', $redirectPath, http_build_query($queryParams)
        ));
        exit;
    }

    public function resetValidation($request)
    {
        $domainName = trim($_SERVER['SERVER_NAME'], 'www.');
        $formData   = $request->json;
        $flag       = true;
        if (empty($formData) || empty($formData['confirmtoken']) || empty($formData['t']) || empty($formData['email']))
        {
            $flag = false;
        }
        else
        {
            if (trim($formData['confirmtoken']) !== trim(md5($domainName . ' ' . $formData['email'])))
            {
                $flag = false;
            }
            $request_time = date('m/d/Y m:i:s', $formData['t']);
            $current_time = date('m/d/Y m:i:s', time());
            $start_date   = new \DateTime($request_time);
            $since_start  = $start_date->diff(new \DateTime($current_time));
            if ($since_start->i > 10)
            {
                $flag = false;
            }
        }
        if ($flag)
        {
            setcookie('reset_link', $formData['email']);
            return ['status' => 'success', 'message' => ''];
        }
        else
        {
            setcookie('reset_link', $flag);
            return ['status' => false, 'message' => 'Link has been expired.'];
        }
    }

    public function pumpOneLandingDashboard($request)
    {
        $email  = $_SESSION['access_token']['user'];
        $userId = UserAccess::getUserIdByEmail($email);

        Config::set('current_step', $this->getCurrentStep());
        Config::set('current_page', 'pumpon_landing');

        $token                       = $this->templateToken($request);
        $token['user_pumpone_token'] = (new \Framework\Modules\PumpOneSSO())->login($email, $userId);
        return $this->templateEngine->loadview('pumpone.html', $token);
    }

    public function paypalResponse($request){
        if($request->get('errorFound')!=='0' && !empty($request->get('orderId'))){
            $redirectPath               = $_SESSION['full_order_data']['current_url'];
            if(empty($request->get('error'))){
                $queryParams['error'] = 'Something went wrong';           
            }
        }
        else{
            $redirectPath               = $_SESSION['full_order_data']['next_url'];
            $queryParams['order_id']    = $request->get('orderId');
            $queryParams['customer_id'] = $request->get('customerId');
        }
        header(sprintf(
            'Location: %s?%s', $redirectPath, http_build_query($queryParams)));
        exit;        
    }    
}
