<?php

namespace Framework\Foundation;

/**
 * The base call for app related info and other configs container
 * All runtime datatype should be added into this class container collection for global access
 * We can access the the container by using app()->container() method from anywhere in the framework.
 */
class Application implements \Framework\Contracts\Application
{

    /**
     * The framework version.
     *
     * @var string
     */
    const VERSION = CMS_BAREBONE_VERSION;

    /**
     * The framework container
     * 
     * @var \Illuminate\Support\Collection
     */
    protected $container = [];
    private static $instances = [];
    protected $crmInstance;

    /**
     * Get the version number of the application.
     *
     * @return string
     */
    public function version()
    {
        return static::VERSION;
    }

    public static function getInstance()
    {
        $cls = static::class;
        if (!isset(self::$instances[$cls]))
        {
            self::$instances[$cls]            = new static();
            self::$instances[$cls]->container = collect();
        }

        return self::$instances[$cls];
    }

    public function bindDefaultConfig($request)
    {
        // Load all runtime application level configurations.
        $this->container = collect(array_merge(app_configs(), [
            'isSecure'      => $request->isSecure(),
            'scheme'        => $request->getScheme(),
            'funnelUri'     => $request->getUri(),
            'funnelBaseUrl' => $this->getBaseUrl($request),
            'funnelPath'    => $request->getPathInfo(), // /path
            'httpUserAgent' => $request->headers->get('User-Agent'),
            'path'          => [
                'base'          => BASE_PATH,
                'storage'       => STORAGE_PATH,
                'customStorage' => CUSTOM_STORAGE_PATH,
            ]
        ]));
    }

    public function getBaseUrl($request)
    {
        $hostName = trim($request->getHttpHost(), '/');
        $basePath = trim($request->getBasePath(), '/');
        $scheme   = $request->getScheme();
        return rtrim(
                sprintf('%s://%s/%s', $scheme, $hostName, $basePath), '/'
        );
    }

    /**
     * Bootstrap the application
     * @param \Symfony\Component\HttpFoundation\Request $request
     */
    public static function bootstrap($request)
    {
        $application = app();
        $application->bindDefaultConfig($request);
    }

    /**
     * The framework container
     * 
     * @return \Illuminate\Support\Collection
     */
    public function container()
    {
        return $this->container;
    }

    public function crmInstance()
    {
        return $this->crmInstance;
    }

    public function setCrmInstance($crm)
    {

        $this->crmInstance = $crm;
    }

    public function isProduction()
    {
        return APP_ENV === 'production';
    }

}
