<?php

namespace Framework\Middleware;

use Closure;
// use Exception;
use Framework\Foundation\Config;
use Framework\Foundation\Helper;
// use Illuminate\Support\Arr;

class LoadDefaultConfiguration implements \Framework\Contracts\Middleware
{
    /**
     * Handle an incoming request.
     *
     * @param  Request  $request
     * @param  \Closure  $next
     * @return mixed
     */
    public function handle($request, Closure $next)
    {
        \Illuminate\Support\Collection::macro('replaceRecursive', function ($items) {
            return new static(array_replace_recursive($this->items, $this->getArrayableItems($items)));
        });

        Config::resetAll(array(
            'template_path'       => sprintf('%s%stemplates', BASE_PATH, DS),
            'route_resolve_error' => 'Something went wrong'
        ));

        $config = Config::get();

        $generalConfig = Helper::getJsonFileData(STORAGE_PATH, 'config.general.json')->replaceRecursive(
                Helper::getJsonFileData(CUSTOM_STORAGE_PATH, 'config.general.json')
        );

        $offersConfig = Helper::getJsonFileData(STORAGE_PATH, 'config.offers.json')->merge(
            Helper::getJsonFileData(CUSTOM_STORAGE_PATH, 'config.offers.json')
        );

        $widget       = Helper::getJsonFileData(STORAGE_PATH, 'widgets.json');
        $errorMessage = Helper::getJsonFileData(STORAGE_PATH, 'error_messages.json')->merge(
            Helper::getJsonFileData(CUSTOM_STORAGE_PATH, 'error_messages.json')
        );
        $shield = Helper::getJsonFileData(STORAGE_PATH, 'shield.json');

        // Merge all together
        $config = $config->merge($generalConfig);
        $config = $config->merge($errorMessage);
        $config = $config->merge(['widget' => $widget]);
        $config = $config->merge(['shield' => $shield]);
        
        $config = $config->put('offer_details', $offersConfig);
        // $config = $config->put('framework_version_history', $this->frameworkVersionHistory());
        Config::resetAll($config);

        return $next($request);
    }

    // private function frameworkVersionHistory() {
    //     try {
    //         $logFileName = 'framework_versions_history';
    //         if (!file_exists(STORAGE_PATH.DS.$logFileName)) {
    //             Helper::createFileifNotExists($logFileName, STORAGE_PATH, app()->version());
    //             return [app()->version()];
    //         }

    //         $versions = explode("\n", file_get_contents(STORAGE_PATH.DS.$logFileName));

    //         if(Arr::last($versions) === app()->version()) return $versions;
            
    //         file_put_contents(STORAGE_PATH.DS.$logFileName, "\n".app()->version(), FILE_APPEND);

    //         return $versions[] = app()->version();
    //     } catch (\Exception $e) {
    //         if(APP_DEBUG) throw new Exception($e->getMessage());

    //         throw new Exception("Something went wrong with the request!");
    //     }
    // }
}
