<?php

namespace Framework\Modules\Crm\Middleware;

use Closure;
use Framework\Foundation\Config;
use Framework\Modules\DeclineManager;

class IsBlockedCustomer implements \Framework\Contracts\Middleware
{
    private $formData, $declineManager;

    public function handle($request, Closure $next)
    {
        $this->formData = $request->json;
        if ($this->isDeclinedManagerEnabled() && $this->isBlockedCustomer())
        {
            # update the DB (For spam customers)
            $this->declineManager->increementCount();
            
            # Set response status to false and add custom error msg
            return array_merge(
                pre_response_array(
                    null, 
                    false, 
                    ['response_message' => $this->declineManager->declineMessage]
                ), ['isActiveCaptcha' => $this->declineManager->showGoogleCaptcha(),]
            );
        }

        return $next($request);
    }

    private function isDeclinedManagerEnabled() {

        if(Config::get('addons.decline-manager.type', 0)) {
            $this->declineManager = new DeclineManager;

            return $this->declineManager->isActive;
        }

        return 0;
    }

    private function isBlockedCustomer() {
        
        $totalDecline = $this->declineManager->totalDecline();
        if (!$totalDecline) return false;

        if($this->resetAfterDateTime() > $this->declineManager->lastDeclinedAt()) {
            $this->declineManager->resetCount();
            return false;
        }

        if($this->isCardDataChanged($this->formData)) {
            $this->declineManager->resetCount();
            $this->declineManager->saveCardData($this->formData);
            return false;
        }

        return ($totalDecline >= $this->declineManager->maxAllowedDecline) ? true : false;
    }

    private function resetAfterDateTime() {
        $mins = $this->declineManager->resetCounterAfterMins;
        return strtotime("-".$mins."minutes");
    }

    private function isCardDataChanged($formData) {

        if(!$this->declineManager->resetCounterAfterCardUpdate) {
            return false;
        }
        
        $card_first_six = \Illuminate\Support\Str::substr(array_get($formData, 'cc_number'), 0,6);
        $card_last_four = \Illuminate\Support\Str::substr(array_get($formData, 'cc_number'), -4);
        $card_exp = sprintf('%s%s', array_get($formData, 'expmonth'), array_get($formData, 'expyear'));
        
        return $this->declineManager->isCardChanged(compact('card_first_six', 'card_last_four', 'card_exp'));
    }

}
