<?php
namespace Framework\Modules\Crm\Middleware;

use Closure;
use Exception;
use Framework\Foundation\Config;
use Framework\Foundation\Helper;
use Framework\Modules\MaxMind;

class MaxMindScoreCheck implements \Framework\Contracts\Middleware
{
    private $isFraud = false;
    private $maxMindConfig;
    private $maxMindscore;

    public function handle($request, Closure $next)
    {
        try{
            $this->maxMindConfig = Config::get('addons.maxmind-score');

            if(!$this->maxMindConfig) return $next($request);

            $this->maxMindscore = new MaxMind(
                $this->maxMindConfig['end_point'], $this->maxMindConfig['account_id'], $this->maxMindConfig['license_key']
            );
            $response = $this->maxMindscore->getMaxMindFraudResponse($this->prepareshippingAddress(),$this->preparebillingAddress());
            
            if($this->maxMindConfig['use_ip_score'])
            {
                $this->isScoreFailedwithIP($response);
            }
            else{
                $this->isScoreFailed($response);
            }
            if($this->isFraud) {
                if($this->maxMindConfig['show_slider_capcha'])
                {
                    return pre_response_array(null, false, [
                        'response_message' => $this->maxMindConfig['err_msg'], 'isSliderCaptcha' => $this->maxMindConfig['show_slider_capcha']
                    ]);
                }
                else{
                    return pre_response_array(null, false, [
                        'response_message' => $this->maxMindConfig['err_msg'], 'isActiveCaptcha' => $this->maxMindConfig['show_google_capcha']
                    ]);
                }
                
            }

        }
        catch(\Exception $e){
            throw new Exception($e->getMessage() . " in file: ". $e->getFile() . " on line: " . $e->getLine());
        }
        
        return $next($request);
    }

    private function prepareshippingAddress(){
        $data = array(
            'first_name'         => array_get($_SESSION, 'prospect_shipping_data.firstName', ''),
            'last_name'          => array_get($_SESSION, 'prospect_shipping_data.lastName', ''),
            'address'            => array_get($_SESSION, 'prospect_shipping_data.address', ''),
            'city'               => array_get($_SESSION, 'prospect_shipping_data.city', ''),
            'region'             => Helper::getStateName(array_get($_SESSION, 'prospect_shipping_data.state', '')),
            'country'            => Helper::getCountryName(array_get($_SESSION, 'prospect_shipping_data.country', '')),
            'postal'             => array_get($_SESSION, 'prospect_shipping_data.zip', ''),
            'phone_number'       => array_get($_SESSION, 'prospect_shipping_data.phone', ''),
        );
        return $data;
    }

    private function preparebillingAddress(){
        
        if(!empty(array_get($_SESSION, 'checkout_billing_data.billingFirstName', ''))){
            $data = array(
                'first_name'         => array_get($_SESSION, 'checkout_billing_data.billingFirstName', ''),
                'last_name'          => array_get($_SESSION, 'checkout_billing_data.billingLastName', ''),
                'address'            => array_get($_SESSION, 'checkout_billing_data.billingAddress', ''),
                'city'               => array_get($_SESSION, 'checkout_billing_data.billingCity', ''),
                'region'             => Helper::getStateName(array_get($_SESSION, 'checkout_billing_data.billingState', '')),
                'country'            => Helper::getCountryName(array_get($_SESSION, 'checkout_billing_data.billingCountry', '')),
                'postal'             => array_get($_SESSION, 'checkout_billing_data.billingZip', ''),
                'phone_number'       => array_get($_SESSION, 'prospect_shipping_data.phone', ''),
            );
        }
        else
        {
            $data = array();
        }
        
        return $data;
    }

    private function isScoreFailed($response)
    {
        if($response['risk_score'] < $this->maxMindConfig['fraud_score'])
        {
            return $this->isFraud = false;
        }
        else{
            return $this->isFraud = true;
        }
    }

    private function isScoreFailedwithIP($response)
    { 
        if($response['risk_score'] < $this->maxMindConfig['fraud_score'] && $response['ip_address']['risk'] < $this->maxMindConfig['ip_fraud_score'])
        {
            return $this->isFraud = false;
        }
        else{
            return $this->isFraud = true;
        }
    }

}
