<?php

namespace Framework\Modules\Crm\Providers;

use Framework\Foundation\Config;
use Framework\Foundation\Helper;
use Framework\Modules\Crm\Contracts\Crm as CrmContracts;

class DemoCRM extends Crm implements CrmContracts
{

    protected $api_base_url;
    protected $username;
    protected $password;
    protected $campaignId;
    protected $shippingId;
    protected $forceGatewayId;
    protected $selectedProducts;
    protected $ipAddress;
    protected $notes;
    protected $httpClient;
    protected static $membershipMethods = [
        'order_update',
        'order_find',
        'order_view',
        'prospect_update',
        'prospect_find',
        'prospect_view',
        'get_alternative_provider',
        'coupon_validate'
    ];
    protected static $formKeys          = [
        'NewProspect'          => [
            'address' => 'address1'
        ],
        'NewOrderWithProspect' => [
            'billingAddress' => 'billingAddress1',
            'cc_type'        => 'creditCardType',
            'cc_number'      => 'creditCardNumber',
            'cvv'            => 'CVV',
            'promo_code'     => 'promoCode'
        ],
        'NewOrder'             => [
            'address'        => 'shippingAddress1',
            'city'           => 'shippingCity',
            'country'        => 'shippingCountry',
            'state'          => 'shippingState',
            'zip'            => 'shippingZip',
            'billingAddress' => 'billingAddress1',
            'cc_type'        => 'creditCardType',
            'cc_number'      => 'creditCardNumber',
            'cvv'            => 'CVV',
            'promo_code'     => 'promoCode'
        ]
    ];

    public function __construct()
    {
        parent::__construct();
    }

    protected function prepareEndpointApi($api_base_url, $method)
    {
        $endpointUrl = $api_base_url;
        return $endpointUrl;
    }

    protected function httpRequest($api, $new_form_params)
    {
        $form_params = [
            'username' => $this->username,
            'password' => $this->password
        ];

        $form_params = array_merge_recursive($form_params, $new_form_params);
        $form_params = array_filter($form_params);

        $response = $this->httpClient->post($api, [
            'body' => $form_params
        ]);

        return $response;
    }

    protected function callApi($method, $form_post_params)
    {
        $api                        = $this->prepareEndpointApi($this->api_base_url, $method);
        $form_post_params['method'] = $method;

        try
        {
            $response = $this->httpRequest($api, $form_post_params);

            $httpResponse = $response->getStatusCode();
            if ($httpResponse == 200)
            {
                $apiResponse = $response->getBody()->getContents();
                parse_str($apiResponse, $apiResponseArr);

                if (!empty($apiResponseArr['response_code']))
                {
                    $apiResponseCode = $apiResponseArr['response_code'];
                }
                else if (!empty($apiResponseArr['responseCode']))
                {
                    $apiResponseCode = $apiResponseArr['responseCode'];
                }

                if (!empty($apiResponseArr['declineReason']))
                {
                    $apiResponseMessage = $apiResponseArr['declineReason'];
                }
                else if (!empty($apiResponseArr['error_message']))
                {
                    $apiResponseMessage = $apiResponseArr['error_message'];
                }
                else if (!empty($apiResponseArr['errorMessage']))
                {
                    $apiResponseMessage = $apiResponseArr['errorMessage'];
                }
                else
                {
                    $apiResponseMessage = '';
                }

                $apiResponseArr['response_message'] = $apiResponseMessage;

                if ($apiResponseCode == 100)
                {
                    // Successful
                    if ($method == 'NewOrder')
                    {
                        $apiResponseArr['prospectId'] = rand(1111, 9999);
                    }

                    return ['status' => true, 'api_response' => $apiResponseArr];
                }
                else
                {
                    // Error
                    return ['status' => false, 'api_response' => $apiResponseArr];
                }
            }
            else
            {
                $apiResponseArr = ['response_message' => 'The request couldn\'t be completed! Please try again later'];
                return ['status' => false, 'api_response' => $apiResponseArr];
            }
        }
        catch (\Exception $ex)
        {
            print_r($ex->getMessage());
            die();
            $apiResponseArr = ['response_message' => 'Something went wrong while processing the request'];
            return ['status' => false, 'api_response' => $apiResponseArr];
        }
    }

    public function newProspect($formData)
    {
        $method     = 'NewProspect';
        $configData = [];

        $configData['campaignId'] = $this->campaignId;
        $configData['ipAddress']  = $this->ipAddress;

        $crmData = array_merge_recursive($formData, $configData);
        $crmData = Helper::renameArrayKey($crmData, self::$formKeys[$method]);
        $crmData = array_filter($crmData);

        $response = $this->callApi($method, $crmData);

        return $response;
    }

    public function newOrderWithProspect($checkoutData)
    {

        $method     = 'NewOrderWithProspect';
        $configData = [];

        if (!empty($checkoutData['expmonth']) && !empty($checkoutData['expyear']))
        {
            $expirationDate = $checkoutData['expmonth'] . $checkoutData['expyear'];
            unset($checkoutData['expmonth']);
            unset($checkoutData['expyear']);
        }
        else
        {
            $expirationDate = '';
        }

        foreach ($this->selectedProducts as $selectedProduct)
        {
            $productIdArr[]                                                              = $selectedProduct['crm_product_id'];
            $checkoutData['product_qty_' . $selectedProduct['crm_product_id']]           = $selectedProduct['crm_product_qty'];
            $checkoutData['dynamic_product_price_' . $selectedProduct['crm_product_id']] = $selectedProduct['crm_product_price'];
        }

        $checkoutData['productId'] = implode(",", $productIdArr);

        $configData['campaignId']     = $this->campaignId;
        $configData['shippingId']     = $this->shippingProfile->get('shipping_profile_crm_id');
        $configData['expirationDate'] = $expirationDate;
        $configData['tranType']       = 'Sale';
        if ($this->forceGatewayId > 0)
        {
            $configData['forceGatewayId']         = $this->forceGatewayId;
            $configData['preserve_force_gateway'] = 1;
        }
        $configData['upsellCount'] = 0;

        $crmData = array_merge_recursive($checkoutData, $configData);
        $crmData = Helper::renameArrayKey($crmData, self::$formKeys[$method]);
        $crmData = array_filter($crmData);

        $response = $this->callApi($method, $crmData);

        return $response;
    }

    public function placeSplitOrder($request, $step)
    {

        $splitChargeDetails = Config::get(sprintf('offer_details.step%s.splitChargeDetails', $step));

        $selectedProducts = array_get($_SESSION, 'external_data.product_data', array_get($splitChargeDetails, 'selectedProduct'));

        $configData = [
            'selectedProducts' => $selectedProducts,
            'campaignId'       => array_get($_SESSION, 'external_data.campaign_id', array_get($splitChargeDetails, 'campaign_id')),
            'shippingId'       => $this->shippingProfile->get('shipping_profile_crm_id'),
            'forceGatewayId'   => array_get($splitChargeDetails, 'crm_force_gateway_id'),
        ];

        return $this->newOrder($request, $configData, 'SplitOrder');
    }

    public function placeUpsellOrder($request, $step)
    {

        $offerConfig      = Config::get('offer_details.step' . $step);
        $selectedProducts = array_get($_SESSION, 'external_data.product_data', array_get($offerConfig, 'selectedProduct'));

        $configData = [
            'selectedProducts' => $selectedProducts,
            'campaignId'       => array_get($_SESSION, 'external_data.campaign_id', array_get($offerConfig, '.campaign_id')),
            'shippingId'       => $this->shippingProfile->get('shipping_profile_crm_id'),
            'forceGatewayId'   => array_get($offerConfig, 'crm_force_gateway_id'),
        ];

        return $this->newOrder($request, $configData, 'UpsellOrder');
    }

    public function placeNewOrder($request, $step)
    {

        $offerConfig      = Config::get('offer_details.step' . $step);
        $selectedProducts = array_get($_SESSION, 'external_data.product_data', array_get($offerConfig, 'selectedProduct'));

        $configData = [
            'selectedProducts' => $selectedProducts,
            'campaignId'       => array_get($_SESSION, 'external_data.campaign_id', array_get($offerConfig, 'campaign_id')),
            'shippingId'       => $this->shippingProfile->get('shipping_profile_crm_id'),
            'forceGatewayId'   => array_get($offerConfig, 'crm_force_gateway_id'),
        ];

        return $this->newOrder($request, $configData, 'NewOrder');
    }

    public function newOrder($orderData, $configData, $type)
    {
        $method = 'NewOrder';

        if (!empty($orderData['expmonth']) && !empty($orderData['expyear']))
        {
            $expirationDate = $orderData['expmonth'] . $orderData['expyear'];
            unset($orderData['expmonth']);
            unset($orderData['expyear']);
        }
        else
        {
            $expirationDate = '';
        }

        $orderData['expirationDate'] = $expirationDate;

        foreach ($this->selectedProducts as $selectedProduct)
        {
            $productIdArr[]                                                           = $selectedProduct['crm_product_id'];
            $orderData['product_qty_' . $selectedProduct['crm_product_id']]           = $selectedProduct['crm_product_qty'];
            $orderData['dynamic_product_price_' . $selectedProduct['crm_product_id']] = $selectedProduct['crm_product_price'];
        }

        $orderData['productId'] = implode(",", $productIdArr);

        $configData['campaignId'] = $this->campaignId;
        $configData['shippingId'] = $this->shippingProfile->get('shipping_profile_crm_id');

        unset($configData['selectedProducts']);

        if ($type == 'SplitOrder' || $type == 'NewOrder' || $type == 'UpsellOrder')
        {
            $configData['upsellCount'] = 0;
            $orderData['productId']    = implode(",", $productIdArr);
        }

        $configData['tranType']  = 'Sale';
        $configData['ipAddress'] = $this->ipAddress;
        $configData['notes']     = $this->notes;

        if (isset($configData['forceGatewayId']) && $configData['forceGatewayId'] > 0)
        {
            $configData['preserve_force_gateway'] = 1;
        }
        else
        {
            unset($configData['forceGatewayId']);
        }

        $crmData = array_merge_recursive($orderData, $configData);
        $crmData = Helper::renameArrayKey($crmData, self::$formKeys[$method]);
        $crmData = array_filter($crmData);

        $response = $this->callApi($method, $crmData);

        return $response;
    }

    public function orderView($queryStrings, $orderIds)
    {
        $method              = 'order_view';
        $crmData['order_id'] = implode(',', $orderIds);

        $response = $this->callApi($method, $crmData);

        if (!empty($response['api_response']['data']))
        {
            $apiResponseArr = json_decode($response['api_response']['data'], true);
        }
        else
        {
            $apiResponseArr = $response['api_response'];
        }

        return $apiResponseArr;
    }

    public function validateCoupon($couponData, $p2)
    {
        $method     = 'coupon_validate';
        $configData = [];

        foreach ($this->selectedProducts as $selectedProduct)
        {
            $productIdArr[]                                                  = $selectedProduct['crm_product_id'];
            $couponData['product_qty_' . $selectedProduct['crm_product_id']] = $selectedProduct['crm_product_qty'];
        }

        $couponData['product_ids'] = implode(",", $productIdArr);

        $configData['campaign_id'] = $this->campaignId;
        $configData['shipping_id'] = $this->shippingProfile->get('shipping_profile_price');

        $crmData = array_merge_recursive($couponData, $configData);
        $crmData = array_filter($crmData);

        $response = $this->callApi($method, $crmData);

        return $response;
    }

    /*
     * @param checkoutData['previousOrderId'=>(int),
     * 					'productId'=>(int),
      'campaignId'=>(int),
      'shippingId'=>(int),
      'product_qty_1'=>(int)
      ] //required
     */

    public function remoteUpsell($checkoutData)
    {
        $method   = 'NewOrderCardOnFile';
        $crmData  = $checkoutData;
        $crmData  = array_filter($crmData);
        $response = $this->callApi($method, $crmData);

        return $response;
    }

    public function isDecline($response)
    {
        return FALSE;
    }

    public function singleOrderView($formData)
    {
        
    }

    public function hasOwnedThisOrder($data, $formData)
    {
        
    }

    public function prepareDataForCancel($data, $formData)
    {
        
    }

    public function cancelOrder($formData)
    {
        
    }

}
