<?php

namespace Framework\Modules\Crm\Providers;

use Framework\Foundation\Config;
use GuzzleHttp\Client;
use Framework\Modules\Crm\Contracts\Crm as CrmContracts;
use Framework\Foundation\Helper;

class Nmi extends Crm implements CrmContracts
{

    const API_ENDPOINT   = "https://secure.networkmerchants.com/api/transact.php";
    const ERROR_RESPONSE = "The request couldn't be completed! Please try again later";

    private $uniApiEndpoint;
    private static $nmiTestCards  = ['4111111111111111'];
    private static $errorMessages = array(
        100 => 'Transaction was approved.',
        200 => 'Transaction was declined by processor.',
        201 => 'Do not honor.',
        202 => 'Insufficient funds.',
        203 => 'Over limit.',
        204 => 'Transaction not allowed.',
        220 => 'Incorrect payment information.',
        221 => 'No such card issuer.',
        222 => 'No card number on file with issuer.',
        223 => 'Expired card.',
        224 => 'Invalid expiration date.',
        225 => 'Invalid card security code.',
        226 => 'Invalid PIN.',
        240 => 'Call issuer for further information.',
        250 => 'Pick up card.',
        251 => 'Lost card.',
        252 => 'Stolen card.',
        253 => 'Fraudulent card.',
        260 => 'Declined with further instructions available. (See response text)',
        261 => 'Declined-Stop all recurring payments.',
        262 => 'Declined-Stop this recurring program.',
        263 => 'Declined-Update cardholder data available.',
        264 => 'Declined-Retry in a few days.',
        300 => 'Transaction was rejected by gateway.',
        400 => 'Transaction error returned by processor.',
        410 => 'Invalid merchant configuration.',
        411 => 'Merchant account is inactive.',
        420 => 'Communication error.',
        421 => 'Communication error with issuer.',
        430 => 'Duplicate transaction at processor.',
        440 => 'Processor format error.',
        441 => 'Invalid transaction information.',
        460 => 'Processor feature not available.',
        461 => 'Unsupported card type.',
    );

    public function __construct()
    {
        parent::__construct();
        $this->funnelId       = Helper::getFunnelId();
        $this->uniApiEndpoint = sprintf("%s/%s/", app()->container()->get('apiEndpoint'), "transactions");
    }

    public function newProspect($formData)
    {
        $data['funnelId']  = $this->funnelId;
        $data['firstName'] = $formData['firstName'];
        $data['lastName']  = $formData['lastName'];
        $data['phone']     = $formData['phone'];
        $data['email']     = $formData['email'];
        $data['address']   = array(
            'address1' => !empty($formData['address']) ? $formData['address'] : '',
            'address2' => !empty($formData['address2']) ? $formData['address2'] : '',
            'city'     => !empty($formData['city']) ? $formData['city'] : '',
            'country'  => !empty($formData['country']) ? $formData['country'] : '',
            'state'    => !empty($formData['state']) ? $formData['state'] : '',
            'zip'      => !empty($formData['zip']) ? $formData['zip'] : '',
        );
        $this->beforeAnyMethodCall($formData);
        $resource          = 'customers';
        $RequestUrL        = sprintf($this->uniApiEndpoint . '%s', $resource);
        $response          = $this->httpRequest($RequestUrL, $data);
        if ($response->getStatusCode() !== 200)
        {
            return false;
        }
        $responseArray = json_decode($response->getBody()->getContents(), true);
        if (!$responseArray['success'])
        {
            $arrayResult = array(
                'status' => $responseArray['success'],
                'errors' => $responseArray['errors'],
            );
            return ['status' => false, 'api_response' => $arrayResult];
        }
        $arrayResult = array(
            'status'     => $responseArray['success'],
            'prospectId' => $responseArray['result']['customerId'],
        );
        return ['status' => true, 'api_response' => $arrayResult];
    }

    private function dataSet($formData)
    {
        $data = array(
            'address1' => $formData['billingAddress'],
            'city'     => $formData['billingCity'],
            'country'  => $formData['billingCountry'],
            'state'    => $formData['billingState'],
            'zip'      => $formData['billingZip'],
        );
        if (!empty($formData['billingAddress2']))
        {
            $data['address2'] = $formData['billingAddress2'];
        }
        return $data;
    }

    public function newOrderWithProspect($formData)
    {
        $this->beforeAnyMethodCall($formData);
        $nmiApiResponse                = $this->performSale();
        $productDetails                = $this->selectedProducts;
        $data                          = $this->productDataSet($productDetails);
        $data['customerId']            = $formData['prospectId'];
        $data['funnelId']              = $this->funnelId;
        $data['billingFirstName']      = $formData['billingFirstName'];
        $data['billingLastName']       = $formData['billingLastName'];
        $check                         = $data['billingSameAsShipping'] = $formData['billingSameAsShipping'];
        if ($check == 'no')
        {
            $data['billingAddress'] = $this->dataSet($formData);
        }
        $nmiEndCheck = $nmiApiResponse['api_response']['is_declined'];
        if ($nmiEndCheck == 0 || $nmiEndCheck == 1)
        {
            $data['transactionId']          = $nmiApiResponse['api_response']['transactionid'];
            $data['isDeclined']             = $nmiApiResponse['api_response']['is_declined'];
            $data['transactionReferenceId'] = $nmiApiResponse['api_response']['transaction_reference_id'];
            $data['gatewayResponse']        = $nmiApiResponse['api_response']['gateway_response'];
        }
        //print_r($data);exit;
        return $this->returnResponse($nmiEndCheck, $data, $check, $formData, $nmiApiResponse);
    }

    protected function httpRequest($requestUrL, $formPostParams)
    {
        $response = $this->httpClient->post($requestUrL, [
            'body'    => json_encode($formPostParams),
            'headers' => [
                'Content-Type' => 'application/json',
                'Api-Token'    => Config::get('uni_api_token'),
            ],
        ]);
        return $response;
    }

    public function productDataSet($productDetails)
    {
        if (!empty($productDetails) && count($productDetails) > 0)
        {
            $data['shippingId']    = $this->shippingProfile->get('shipping_profile_crm_id');
            $data['shippingPrice'] = $this->shippingProfile->get('shipping_profile_price');
            foreach ($productDetails as $key => $val)
            {
                $data['productDetails'][] = array(
                    'productId'       => !empty($val['product_id']) ? $val['product_id'] : '',
                    'productPrice'    => $val['crm_product_price'],
                    'productQuantity' => $val['crm_product_qty'],
                );
            }
        }
        return $data;
    }

    public function newOrder($formData, $configData, $type)
    {
        $this->beforeAnyMethodCall($formData);
        $nmiApiResponse = $this->performSale();
        $productDetails = $this->selectedProducts;
        $data           = $this->productDataSet($productDetails);
        if (!empty($configData['customer_id']))
        {
            $data['customerId'] = $configData['customer_id'];
        }
        $data['funnelId']              = $this->funnelId;
        $data['firstName']             = $formData['firstName'];
        $data['lastName']              = $formData['lastName'];
        $data['phone']                 = $formData['phone'];
        $data['email']                 = $formData['email'];
        $data['address']               = array(
            'address1' => !empty($formData['address']) ? $formData['address'] : '',
            'address2' => !empty($formData['address2']) ? $formData['address2'] : '',
            'city'     => !empty($formData['city']) ? $formData['city'] : '',
            'country'  => !empty($formData['country']) ? $formData['country'] : '',
            'state'    => !empty($formData['state']) ? $formData['state'] : '',
            'zip'      => !empty($formData['zip']) ? $formData['zip'] : '',
        );
        $check                         = $data['billingSameAsShipping'] = $formData['billingSameAsShipping'];
        if ($check == 'no')
        {
            $data['billingAddress'] = $this->dataSet($formData);
        }
        if (!empty($formData['billingFirstName']) && !empty($formData['billingLastName']))
        {
            $data['billingFirstName'] = $formData['billingFirstName'];
            $data['billingLastName']  = $formData['billingLastName'];
        }
        $nmiEndCheck = $nmiApiResponse['api_response']['is_declined'];
        if ($nmiEndCheck == 0 || $nmiEndCheck == 1)
        {
            $data['transactionId']          = $nmiApiResponse['api_response']['transactionid'];
            $data['isDeclined']             = $nmiApiResponse['api_response']['is_declined'];
            $data['transactionReferenceId'] = $nmiApiResponse['api_response']['transaction_reference_id'];
            $data['gatewayResponse']        = $nmiApiResponse['api_response']['gateway_response'];
        }
        return $this->returnResponse($nmiEndCheck, $data, $check, $formData, $nmiApiResponse);
    }

    private function returnResponse($nmiEndCheck, $data, $check, $formData, $nmiApiResponse)
    {
        if ($nmiEndCheck == 0)
        {
            $resource = 'orders';
            $url      = sprintf($this->uniApiEndpoint . '%s', $resource);
            $check == 'no';
            $response = $this->httpRequest($url, $data);

            if ($response->getStatusCode() !== 200)
            {
                return false;
            }
            $responseArray = json_decode($response->getBody()->getContents(), true);
            return $this->returnResponseFinal($responseArray, $formData, $response);
        }
        elseif ($nmiEndCheck == 1)
        {
            return ['status' => false, 'api_response' => $nmiApiResponse['api_response']];
        }
        else
        {
            return ['status' => false, 'api_response' => 'Something went wrong'];
        }
    }

    private function returnResponseFinal($responseArray, $formData, $response)
    {
        if (!$responseArray['success'])
        {
            return ['status' => false, 'api_response' => ['response_message' => !empty($responseArray['errors']['message']) ? $responseArray['errors']['message'] : $responseArray['errors'][0]['message']]];
        }
        $orderResponse                                     = array(
            'status'     => $responseArray['success'],
            'orderId'    => $responseArray['result']['orderId'],
            'customerId' => !empty($responseArray['result']['customerId']) ? $responseArray['result']['customerId'] : $formData['prospectId'],
        );
        $_SESSION['nmi_orders'][$orderResponse['orderId']] = $response;
        return ['status' => true, 'api_response' => $orderResponse];
    }

    public function orderView($queryStrings, $orderIds)
    {
        $data = array();
        foreach ($orderIds as $orderId)
        {
            if (!empty($_SESSION['nmi_orders'][$orderId]))
            {
                $data[$orderId] = $_SESSION['nmi_orders'][$orderId];
            }
            else
            {
                $data[$orderId] = 'Order not found!';
            }
        }

        return $data;
    }

    private function performSale()
    {
        $params = array_replace_recursive(
                $this->getCardDetails(),
                $this->getPriceAndShippingAmount(),
                $this->getShippingDetails(),
                $this->getBillingDetails()
        );

        $params['type']    = 'sale';
        $params['orderid'] = strtoupper(uniqid());

        if ($this->makeRequest($params))
        {
            unset($this->response['response_message']);
        }
        return $this->response;
    }

    private function getShippingDetails()
    {
        return array_filter(array(
            'shipping_firstname' => $this->customerData['firstName'],
            'shipping_lastname'  => $this->customerData['lastName'],
            'shipping_address1'  => $this->customerData['address'],
            'shipping_city'      => $this->customerData['city'],
            'shipping_state'     => $this->customerData['state'],
            'shipping_zip'       => $this->customerData['zip'],
            'shipping_country'   => $this->customerData['country'],
            'shipping_phone'     => $this->customerData['phone'],
            'shipping_email'     => $this->customerData['email'],
        ));
    }

    private function getBillingDetails()
    {
        if (
                strtolower($this->customerData['billingSameAsShipping']) !== 'no'
        )
        {
            return array_filter(array(
                'firstname' => $this->customerData['firstName'],
                'lastname'  => $this->customerData['lastName'],
                'address1'  => $this->customerData['address'],
                'city'      => $this->customerData['city'],
                'state'     => $this->customerData['state'],
                'zip'       => $this->customerData['zip'],
                'country'   => $this->customerData['country'],
            ));
        }
        return array_filter(array(
            'firstname' => $this->customerData['billingFirstName'],
            'lastname'  => $this->customerData['billingLastName'],
            'address1'  => $this->customerData['billingAddress'],
            'city'      => $this->customerData['billingCity'],
            'state'     => $this->customerData['billingState'],
            'zip'       => $this->customerData['billingZip'],
            'country'   => $this->customerData['billingCountry'],
        ));
    }

    private function getPriceAndShippingAmount()
    {
        $data = array('amount' => 0.00, 'shipping' => 0.00, 'tax' => 0.00);

        if (!is_array($this->selectedProducts))
        {
            return $data;
        }

        foreach ($this->selectedProducts as $product)
        {
            if (!empty($product['crm_product_price']))
            {
                $data['amount'] += (float) $product['crm_product_price'];
            }
            if (!empty($product['crm_product_shipping_price']))
            {
                $data['shipping'] += (float) $product['crm_product_shipping_price'];
            }
        }

        return array_map(function ($value)
        {
            return number_format($value, 2);
        }, $data);
    }

    private function getCardDetails()
    {
        return array_filter(array(
            'ccnumber' => $this->customerData['cc_number'],
            'ccexp'    => sprintf(
                    '%s%s', $this->customerData['expmonth'],
                    $this->customerData['expyear']
            ),
            'cvv'      => $this->customerData['cvv'],
        ));
    }

    private function makeRequest($params)
    {

        /**
         * Checking for simulate live transation (simulate_live_trans === 1)
         * Is test cart === false
         * If true, then return true and set response as below format (dummey response)
         * Is card Valid
         */
        if (
                Config::get('payment_provider.simulate_live_trans') &&
                !in_array($params['ccnumber'], static::$nmiTestCards) &&
                Helper::validateCard($params['ccnumber'])
        )
        {
            $tid            = time();
            $this->response = [
                'status'       => 1,
                'api_response' => [
                    'is_declined'              => 0,
                    'gateway_response'         => 'Test Order.',
                    'transaction_reference_id' => $params['orderid'],
                    'transactionid'            => $tid,
                    'sim_live_trans'           => true
                ]
            ];
            return true;
        }

        $params = array_replace_recursive($params, array(
            'username'  => $this->username,
            'password'  => $this->password,
            'ipaddress' => Helper::getIP(),
        ));

        $url = sprintf('%s?%s', self::API_ENDPOINT, http_build_query($params));

        $httpClient = new Client();
        $httpClient->setDefaultOption('verify', false);
        $response   = $httpClient->get($url);
        if ($response->getStatusCode() !== 200)
        {
            return false;
        }

        parse_str($response->getBody()->getContents(), $responseArray);
        $gateway_response                  = $httpClient->get($url)->getBody()->getContents();
        $responseArray['gateway_response'] = $gateway_response;
        return $this->prepareResponse($responseArray);
    }

    private function prepareResponse($response)
    {
        if (empty($response['response']) || $response['response'] != 1)
        {
            $this->response['status'] = false;

            if (!empty(self::$errorMessages[$response['response_code']]))
            {
                $this->response['api_response'] = array(
                    'response_message'         => self::$errorMessages[
                    $response['response_code']
                    ],
                    'is_declined'              => 1,
                    'gateway_response'         => $response['gateway_response'],
                    'transaction_reference_id' => $response['orderid'],
                    'transactionid'            => '',
                );
            }

            return false;
        }
        $this->response['status']       = true;
        $this->response['api_response'] = array(
            'is_declined'              => 0,
            'gateway_response'         => $response['gateway_response'],
            'transaction_reference_id' => $response['orderid'],
            'transactionid'            => $response['transactionid'],
        );
        return true;
    }

    private function beforeAnyMethodCall($formData)
    {
        $this->customerData = array();
        if (!empty($_SESSION['prospect_shipping_data']))
        {
            $this->customerData = array_replace_recursive(
                    $this->customerData, $_SESSION['prospect_shipping_data']
            );
        }
        if (!empty($_SESSION['full_order_data']))
        {
            $this->customerData = array_replace_recursive(
                    $this->customerData, $_SESSION['full_order_data']
            );
        }

        $this->customerData = array_replace_recursive(
                $this->customerData, $formData
        );

        $this->response = array(
            'status'       => false,
            'api_response' => array(
                'response_message' => self::ERROR_RESPONSE,
            ),
        );
    }

    public function isDecline($response)
    {
        return !empty($response['api_response']['is_declined']) ? true : false;
    }

    public function singleOrderView($formData)
    {
        
    }

    public function hasOwnedThisOrder($data, $formData)
    {
        
    }

    public function prepareDataForCancel($data, $formData)
    {
        
    }

    public function cancelOrder($formData)
    {
        
    }

}
