<?php

namespace Framework\Modules\Crm\Providers;

use Framework\Foundation\Config;
use Framework\Modules\Crm\Contracts\Crm as CrmContracts;

class Sixcrm extends Crm implements CrmContracts
{

    protected $api_base_url;
    protected $secret_key;
    protected $access_key;
    protected $account_id;
    protected $affiliates;
    protected $acquisitionToken = null;
    protected $campaignId;
    protected $isOrderDeclined  = false;

    # Extra fields
    protected $username;
    protected $password;
    protected $shippingId;
    protected $forceGatewayId;
    protected $selectedProducts;
    protected $ipAddress;
    protected $notes;
    protected $httpClient;

    public function __construct()
    {
        parent::__construct();
        $this->_step      = 1;
        $this->account_id = Config::get('api_account_id');
        # Optional Fields
        $this->affiliates = Config::get('affiliates');
        if (is_null($this->affiliates))
        {
            $this->affiliates = [];
        }

        # Extra Fields
        $this->selectedProducts = $this->_getAllProducts();

        # Get the CRM auth key
        $this->_acquireToken();
    }

    # 'lead/create/'

    public function newProspect($formData)
    {
        try
        {
            $crmResponse = $this->_callApi(
                    'post',
                    'prospect',
                    $this->_prepareProspectBody($formData)
            );

            if ($crmResponse['success'] && $crmResponse['code'] === 200)
            {
                # set some session variable for order process
                $this->_setAdditionalSessionData($crmResponse);
                # Return result
                return [
                    'status'       => true,
                    'api_response' => $this->_prepareApiResponse('prospect', $crmResponse['response']),
                ];
            }
            else
            {
                return [
                    'status'       => false,
                    'api_response' => $this->_prepareApiResponse('prospect', $crmResponse['response']),
                    'ex_message'   => 'Unable to create prospect!',
                ];
            }
        }
        catch (\Exception $e)
        {
            return ['status' => false, 'api_response' => ['response_message' => $e->getMessage()], 'ex_message' => $e->getMessage()];
        }
    }

    # 'order/create/'

    public function newOrderWithProspect($checkoutData)
    {
        $checkoutData = array_merge($_SESSION['prospect_shipping_data'], $checkoutData);

        $crmData            = [];
        $crmData['session'] = $checkoutData['six_crm_session_id'];

        $crmData['products'] = $this->_prepareProducts();

        $crmData['creditcard']          = $this->_prepareCreditCard($checkoutData, 'newOrderWithProspect');
        $crmData['transaction_subtype'] = "main";

         $crmResponse = $this->_callApi(
                'post',
                'newOrderWithProspect',
                $crmData
        );

        if ($crmResponse['success'] && $crmResponse['code'] === 200 && $crmResponse['response']['result'] !== "error")
        {
            # Return result
            return [
                'status'       => true,
                'api_response' => $this->_prepareApiResponse('newOrderWithProspect', $crmResponse['response']),
            ];
        }
        else
        {
            return [
                'status'       => false,
                'api_response' => ['response_message' => 'Transaction has been declined.',
                    'data'             => $this->_prepareApiResponse('newOrder', $crmResponse['response'])],
                'ex_message'   => 'Unable to place your order!',
            ];
        }
    }

    public function newOrder($request, $configData, $type)
    {
       
        # Update products
        if ($this->_step > 1)
        $this->_updateProductList();

        # Call API end-point
        $crmResponse = $this->_callApi(
                'post',
                'newOrder',
                $this->_prepareSingleCheckoutBody($request)
        );

        if ($crmResponse['success'] && $crmResponse['code'] === 200 && $crmResponse['response']['result'] !== "error")
        {
            # set some session variable for order process
            $this->_setAdditionalSessionData($crmResponse['response']);
            # Return result
            return [
                'status'       => true,
                'api_response' => $this->_prepareApiResponse('newOrder', $crmResponse['response']),
            ];
        }
        else
        {
            return [
                'status'       => false,
                'api_response' => ['response_message' => 'Transaction has been declined.',
                    'data'             => $this->_prepareApiResponse('newOrder', $crmResponse['response'])],
                'ex_message'   => 'Unable to place your order!',
            ];
        }
        
        return ['status' => false, 'api_response' => ['response_message' => "Something went wrong, please contact support!"]];
    }

    public function remoteUpsell($checkoutData)
    {
        // newOrderCardOnFile
    }

    public function orderView($queryStrings, $orderIds)
    {
        try
        {
            $crmResponse = $this->orderConfirm($_SESSION['prospect_shipping_data']['six_crm_session_id']);
            if (!empty($crmResponse['success']))
            {
                if (!is_array($crmResponse['response']))
                {
                    $apiResponseArr = json_decode($crmResponse['response'], true);
                }
                else
                {
                    $apiResponseArr = $crmResponse['response'];
                }
            }
            else
            {
                $apiResponseArr = [];
            }

            return $apiResponseArr;
        }
        catch (\Exception $e)
        {
            return [];
        }
    }

    public function orderConfirm($crmSessionId)
    {
        $endPoint = $this->_prepareEndpointApi('orderConfirm');
        $endPoint .= '?session=';
        $endPoint .= $crmSessionId;
        $headers  = [
            'Content-Type'  => 'application/json',
            'Authorization' => $this->acquisitionToken,
        ];

        $this->httpClient->setDefaultOption('headers', $headers);

        $response = $this->httpClient->get($endPoint);
        return json_decode((string) $response->getBody(), true);
    }

    /*     * ********************* All Helper methods and Attributes ************************ */

    private function _setAdditionalSessionData($crmResponse)
    {
        if (!empty($crmResponse['session']))
        {
            if (!empty($crmResponse['session']['customer']) && !empty($crmResponse['session']['id']))
            {
                if (empty($_SESSION['prospect_shipping_data']))
                {
                    $_SESSION['prospect_shipping_data'] = [];
                }
                $_SESSION['prospect_shipping_data']['six_crm_session_id']  = $crmResponse['session']['id'];
                $_SESSION['prospect_shipping_data']['six_crm_customer_id'] = $crmResponse['session']['customer'];
            }
            else
            {
                throw new \Exception("Invalid CRM response", 9999);
            }
        }
        else if (!empty($crmResponse['response']))
        {
            if (!empty($crmResponse['response']['customer']) && !empty($crmResponse['response']['id']))
            {
                if (empty($_SESSION['prospect_shipping_data']))
                {
                    $_SESSION['prospect_shipping_data'] = [];
                }
                $_SESSION['prospect_shipping_data']['six_crm_session_id']  = $crmResponse['response']['id'];
                $_SESSION['prospect_shipping_data']['six_crm_customer_id'] = $crmResponse['response']['customer'];
            }
            else
            {
                throw new \Exception("Invalid CRM response", 9999);
            }
        }
        else
        {
            throw new \Exception("No CRM api response", 9999);
        }
    }

    private function _prepareApiResponse($requestFor, $crmResponse = [])
    {
        if ($requestFor == 'prospect')
        {
            $crmResponse['prospectId'] = rand(1111, 9999);
        }
        else if ($requestFor == 'newOrderWithProspect' || $requestFor == 'newOrder')
        {

            // Check the order is declined or success
            if (!empty($crmResponse['result']))
            {
                if ($crmResponse['result'] != 'success')
                {
                    $this->isOrderDeclined = true;
                }
            }

            $crmResponse['prospectId'] = !empty($_GET['prospect_id']) ? $_GET['prospect_id'] : rand(1111, 9999);
            if (!empty($_SESSION['prospect_shipping_data']['six_crm_customer_id']))
            {
                $crmResponse['customerId'] = $_SESSION['prospect_shipping_data']['six_crm_customer_id'];
            }
            else
            {
                if ($crmResponse['result'] && !empty($crmResponse['session']))
                {
                    $crmResponse['customerId'] = $crmResponse['session']['customer'];
                }
                else
                {
                    $crmResponse['customerId'] = '';
                }
            }

            # Get Order ID
            if (!empty($crmResponse['order']['id']))
            {
                $orderId = $crmResponse['order']['id'];
            }
            else if (!empty($crmResponse['orders']))
            {
                // For single page checkout
                $orderId = $crmResponse['orders'][0]['id'];
            }
            else
            {
                $orderId = "";
            }

            if (empty($orderId))
            {
                throw new \Exception("Unable to place your order, please try again!", 9999);
            }

            $crmResponse['orderId']                   = $orderId;
            $crmResponse['selectedProducts']          = $this->selectedProducts;
            $_SESSION['all_order'][(string) $orderId] = $crmResponse;
        }

        return $crmResponse;
    }

    private function _callApi($method = 'post', $endPoint = 'tokenAcquire', $acquire_body = [])
    {
        try
        {
            $headers = [
                'Content-Type'  => 'application/json',
                'Authorization' => $this->acquisitionToken,
            ];

            $this->httpClient->setDefaultOption('headers', $headers);
            $httpClientFields = [];

            if (!empty($acquire_body))
            {
                $httpClientFields['body'] = json_encode($acquire_body);
            }

            $response = $this->httpClient->{$method}($this->_prepareEndpointApi($endPoint), $httpClientFields);
            return json_decode((string) $response->getBody(), true);
        }
        catch (\Exception $e)
        {
            throw new \Exception($e->getMessage(), 9999);
        }
    }

    private function _prepareEndpointApi($method)
    {
        $methodSlugMapping = [
            'prospect'             => 'lead/create/',
            'newOrderWithProspect' => 'order/create/',
            'newOrder'             => 'checkout/',
            'newOrderCardOnFile'   => 'order/create/',
            'orderConfirm'         => 'order/confirm/',
            'tokenAcquire'         => 'token/acquire/',
        ];

        if (array_key_exists($method, $methodSlugMapping))
        {
            $endpointUrl = $this->_get('api_base_url') . '/' . $methodSlugMapping[$method] . $this->_get('account_id');
        }
        else
        {
            throw new \Exception("Invalid method name : {$method}", 9999);
        }
        return $endpointUrl;
    }

    private function _prepareSingleCheckoutBody($formData)
    {

        $required = ['firstName', 'lastName', 'address', 'city', 'country', 'state', 'zip', 'phone', 'email', 'agreeCheckbox', "cc_type", "cc_number", "expmonth", "expyear", "cvv", "productId", "productQty"];


        $data = [];
        foreach ($required as $field)
        {
            if (array_key_exists($field, $formData))
            {
                $data[strtolower($field)] = !empty($formData[$field]) ? trim($formData[$field]) : 'N/A';
            }
            else
            {
                $data[strtolower($field)] = 'N/A';
            }
        }

        $payload = [
            'campaign' => $this->campaignId,
            'customer' => [
                'firstname' => $data['firstname'],
                'lastname'  => $data['lastname'],
                'email'     => $data['email'],
                'phone'     => $data['phone'],
                'address'   => $this->_prepareShippingAddress($data),
            ],
        ];

        $getBillingAddress = $this->_prepareBillingAddress($formData);
        if (!empty($getBillingAddress))
        {
            $payload['customer']['billing'] = $getBillingAddress['address'];
        }

        $payload['creditcard'] = $this->_prepareCreditCard($formData);
        $payload['products']   = $this->_prepareProducts();
        return $payload;
    }

    private function _prepareProspectBody($formData)
    {
        $required = ['firstName', 'lastName', 'address', 'city', 'country', 'state', 'zip', 'phone', 'email', 'agreeCheckbox'];
        $data     = [];
        foreach ($required as $field)
        {
            if (array_key_exists($field, $formData))
            {
                $data[strtolower($field)] = !empty($formData[$field]) ? trim($formData[$field]) : 'N/A';
            }
            else
            {
                $data[strtolower($field)] = 'N/A';
            }
        }

        $payload           = [
            'campaign' => $this->campaignId,
            'customer' => [
                'firstname' => $data['firstname'],
                'lastname'  => $data['lastname'],
                'email'     => $data['email'],
                'phone'     => $data['phone'],
                'address'   => $this->_prepareShippingAddress($data),
            ],
        ];
        $getBillingAddress = $this->_prepareBillingAddress($data);
        if (!empty($getBillingAddress))
        {
            $payload['customer']['billing'] = $getBillingAddress['address'];
        }

        return $payload;
    }

    private function _prepareCreditCard($data, $orderType = 'newOrderWithProspect')
    {
        $cardData               = [];
        $cardData['number']     = $data['cc_number'];
        $cardData['expiration'] = (string) $data['expmonth'] . $data['expyear'];
        $cardData['cvv']        = $data['cvv'];
        $cardData['name']       = $data['firstName'] . ' ' . $data['lastName'];

        $address = $this->_prepareShippingAddress($data);
        $billing = $this->_prepareBillingAddress($data);

        if (!empty($billing))
        {
            $cardData['address'] = $billing['address'];
            $cardData['name']    = $billing['customer']['firstName'] . ' ' . $billing['customer']['lastName'];
        }
        else
        {
            $cardData['address'] = $address;
        }

        return $cardData;
    }

    private function _prepareBillingAddress($data)
    {
        $billingDetails = [];
        if (!empty($data['billingSameAsShipping']) && strtolower($data['billingSameAsShipping']) !== 'yes')
        {
            $billingDetails['address'] = array(
                'line1'   => $this->_prepareLine1Address($data['billingAddress']),
                'zip'     => $data['billingZip'],
                'city'    => $data['billingCity'],
                'state'   => $data['billingState'],
                'country' => $data['billingCountry'],
            );

            $billingDetails['customer']['firstName'] = $data['billingFirstName'];
            $billingDetails['customer']['lastName']  = $data['billingLastName'];
        }

        return $billingDetails;
    }

    private function _prepareShippingAddress($data)
    {
        $shippingDetails = [
            'city'    => $data['city'],
            'state'   => $data['state'],
            'zip'     => $data['zip'],
            'country' => $data['country'],
        ];

        if (!empty($data['address']))
        {
            $shippingDetails['line1'] = $this->_prepareLine1Address($data['address']);
        }

        if (!empty($data['address2']))
        {
            $shippingDetails['line2'] = $data['address2'];
        }

        return $shippingDetails;
    }

    private function _prepareLine1Address($line1)
    {
        $addressLine = explode(" ", $line1);
        $line        = '';
        if (count($addressLine) > 1)
        {
            if (preg_match('/^[0-9]+$/', $addressLine[0]))
            {
                $line = $addressLine[0] . ' ' . $addressLine[1];
            }
            else
            {
                throw new \Exception("Street Number is required with address line1", 1);
            }
        }
        else
        {
            throw new \Exception("Street Number is required with address line1", 1);
        }
        return $line;
    }

    private function _getStep()
    {
        if (!empty($_SESSION['upsell_step']))
            $this->_step = (int) $_SESSION['upsell_step'];
    }

    private function _getAllProducts()
    {
        # Setup current step (main=1, upsell1 = 2 etc)
        $this->_getStep();
        return !empty($_SESSION['external_data']['product_data']) ? $_SESSION['external_data']['product_data'] : Config::get('offer_details.step' . $this->_step . '.selectedProduct');
    }

    private function _updateProductList()
    {
        # Update current product list on current step
        $this->selectedProducts = $this->_getAllProducts();
    }

    private function _prepareProducts()
    {
        $result = array();
        if (!empty($this->selectedProducts))
        {
            foreach ($this->selectedProducts as $i => $product)
            {
                if (empty($product['crm_product_id']))
                {
                    continue;
                }

                $obj{$i + 1}           = new \stdClass();
                $obj{$i + 1}->product  = $product['crm_product_id'];
                $obj{$i + 1}->quantity = (int) $product['crm_product_qty'];
                $obj{$i + 1}->price    = $product['crm_product_price'];
                array_push($result, $obj{$i + 1});
            }
        }
        else
        {
            throw new \Exception("No product found in funnel config!", 1);
        }
        return $result;
    }

    /*     * *****-------------------------------------******* */
    # Get a property value

    private function _get($key)
    {
        if (property_exists($this, "$key"))
            return $this->{$key};
        return null;
    }

    private function _acquireToken()
    {
        if (empty($_SESSION['six_crm_acquisitionToken']))
        {
            $signature    = $this->_generateAcquireTokenSignature();
            $account      = $this->_get('account_id');
            $acquire_body = $this->_getAcquireBody();
            $headers      = [
                'Content-Type'  => 'application/json',
                'Authorization' => $signature,
            ];

            $this->httpClient->setDefaultOption('headers', $headers);
            $response = $this->httpClient->post($this->_prepareEndpointApi('tokenAcquire'), [
                'body' => json_encode($acquire_body),
            ]);

            try
            {
                $data = json_decode((string) $response->getBody(), true);
                if ($data['success'] && !empty($data['response']))
                {
                    $this->acquisitionToken               = $_SESSION['six_crm_acquisitionToken'] = trim($data['response']);
                }
                else
                {
                    $this->acquisitionToken               = $_SESSION['six_crm_acquisitionToken'] = null;
                }
            }
            catch (\Exception $e)
            {
                $this->acquisitionToken               = $_SESSION['six_crm_acquisitionToken'] = null;
            }
        }
        else
        {
            $this->acquisitionToken = $_SESSION['six_crm_acquisitionToken'];
        }
    }

    private function _generateAcquireTokenSignature()
    {
        $access_key   = $this->access_key;
        $request_time = time() * 1E3;
        $signature    = sha1($this->secret_key . $request_time);
        return implode(':', array($access_key, $request_time, $signature));
    }

    private function _getAcquireBody()
    {
        $rawBody             = [];
        $rawBody['campaign'] = $this->_get('campaignId');
        if (!empty($this->_get('affiliates')))
        {
            $rawBody['affiliates'] = $this->_get('affiliates');
        }
        return $rawBody;
    }

    public function isDecline($response)
    {
        return $response['result'] != 'success' ? true : false;
    }

    public function singleOrderView($formData)
    {
        
    }

    public function hasOwnedThisOrder($data, $formData)
    {
        
    }

    public function prepareDataForCancel($data, $formData)
    {
        
    }

    public function cancelOrder($formData)
    {
        
    }

}
