<?php

namespace Framework\Modules;

use Framework\Models\Customer;
use Framework\Foundation\Helper;
use Framework\Models\CustomerMeta;
use \Framework\Foundation\Config;

class DeclineManager {

    private $customer;
    public $maxAllowedDecline, $isEnabled, $resetCounterAfterMins, $declineMessage, $type, $isActive, $resetCounterAfterCardUpdate;

    public function __construct()
    {
        // DB migration
        (new Customer)->createMigration();
        (new CustomerMeta)->createMigration();

        $this->customer = Customer::firstOrCreate([
            'ip' => Helper::getIP(),
            'user_agent' =>  md5($_SERVER['HTTP_USER_AGENT'])
        ]);

        $this->declineMessage = Config::get('addons.decline-manager.decline_message');
        $this->type = Config::get('addons.decline-manager.type');
        $this->isActive = Config::get('addons.decline-manager.type', 0) ? 1 : 0;
        $this->maxAllowedDecline = Config::get('addons.decline-manager.maximum_allowed_decline_limit', 5);
        $this->resetCounterAfterMins = Config::get('addons.decline-manager.block_time_duration', 1440); // default is 1 day (1440 mins)
        $this->resetCounterAfterCardUpdate = Config::get('addons.decline-manager.reset_counter_after_card_update', 0);
    }

    public function totalDecline() {

        $count = $this->customer->allMeta()->get('total_declined');

        return !empty($count) ? $count : 0;
    }

    public function lastDeclinedAt() {
        return $this->customer->allMeta()->get('declined_at');
    }

    public function increementCount() {

        $count = $this->customer->allMeta()->get('total_declined');
        return $this->setCount(++$count);

    }

    public function decrementCount($count = null) {

        $count = $this->customer->allMeta()->get('total_declined');
        return $this->setCount($count > 0 ? --$count : 0);
    }

    public function setCount($count) {

        $meta = CustomerMeta::firstOrCreate([
            'customer_id' => $this->customer->id, 
            'meta_key' => 'total_declined'
        ]);
        $meta->meta_value = $count;
        $meta->save();


        $meta = CustomerMeta::firstOrCreate([
            'customer_id' => $this->customer->id, 
            'meta_key' => 'declined_at'
        ]);
        $meta->meta_value = time();
        $meta->save();
    }

    public function resetCount() {
        $this->setCount(0);
        appLog('reset_declined_count', "at ".time(), 1);
    }

    public function saveMetaData(Array $data) {

        foreach ($data as $key => $value) {

            $meta = CustomerMeta::firstOrCreate([
                'customer_id' => $this->customer->id, 
                'meta_key' => $key
            ]);

            $meta->meta_value = $value;
            $meta->save();
        }
    }

    public function isCardChanged(Array $cardData) {

        $meta = $this->customer->allMeta();

        if(
            empty($cardData['card_first_six']) || 
            empty($cardData['card_last_four']) ||
            empty($cardData['card_exp'])
            ) {
            
                return false;
        }
        
        if(
            $meta->get('card_first_six') != $cardData['card_first_six'] || 
            $meta->get('card_last_four') != $cardData['card_last_four'] ||
            $meta->get('card_exp') != $cardData['card_exp']
            ) {
            
                return true;
        }

        return false;
    }

    public function showGoogleCaptcha() {

        $spamCount = $this->totalDecline();
        
        if (!$this->isActive || !$spamCount) {
            return 0;
        }

        return ($spamCount > ($this->maxAllowedDecline + 2)) ? 1 : 0;
    }


    public function saveCardData($formData) {

        $card_first_six = \Illuminate\Support\Str::substr(array_get($formData, 'cc_number'), 0,6);
        $card_last_four = \Illuminate\Support\Str::substr(array_get($formData, 'cc_number'), -4);
        $card_exp = sprintf('%s%s', array_get($formData, 'expmonth'), array_get($formData, 'expyear'));
        
        return $this->saveMetaData(compact('card_first_six', 'card_last_four', 'card_exp'));
    }
}