<?php
namespace Framework\Modules\Migration;

use Framework\Foundation\Helper;

class FunnelConfig {

    private static $funnelJsonData;
    private static $generalConfigs;
    
    public static function migrate() {
        static::$funnelJsonData = Helper::getJsonFileData(STORAGE_PATH, 'funnel.json');
    
        if(static::$funnelJsonData->isEmpty()) return;

        static::mapGeneral();
        static::mapFunnel();
        static::mapDomain();
        static::mapSigner();
        static::mapCompany();
        static::mapPaymentMethods();
        static::mapEvents();
        static::mapAddons();
        static::mapPaymentProvider();

        Helper::createFileifNotExists('config.general.json', STORAGE_PATH, json_encode(static::$generalConfigs, JSON_PRETTY_PRINT));

    }

    private static function filterValue($val) {
        if($val === "") return null;

        return $val;
    }

    private static function mapGeneral() {

        static::$generalConfigs['uni_api_token'] = static::$funnelJsonData->get('elminara_api_token', null);
        static::$generalConfigs['uni_api_endpoint'] = static::$funnelJsonData->get('api_responder_endpoint', null);
    }

    private static function mapFunnel() {
        $map = collect([
            "funnel_id"             => "id",
            "funnel_name"           => "name",
            "funnel_https_status"   => "https_status",
            "funnel_url"            => "url",
            "funnel_support_email"  => "funnel_support_email",
            "funnel_support_phone"  => "funnel_support_phone",
            "funnel_path"           => "path",
            "funnel_type"           => "type",
            "funnel_short_term"     => "short_term",
            "funnel_extended_version" => "extended_version",
            "disable_funnel_status"   => "disable",
            "disable_nav_url"         => "disable_redirect_url",
            "data_collection_disclaimer" => "disclaimer",
            "google_analytics_id"    => "google_analytics_id",
        ]);

        foreach (static::$funnelJsonData as $key => $value) {

            if($map->has($key)) {
                static::$generalConfigs['funnel'][$map->get($key)] = self::filterValue($value);
            }
        }

        static::$generalConfigs['funnel']['countries'] = [
            'available' => static::$funnelJsonData->get('countries', null),
            'is_islands_in_list' => static::$funnelJsonData->get('is_islands_in_list', 0),
            'is_armed_forces_in_list' => static::$funnelJsonData->get('is_armed_forces_in_list', 0),
        ];

        static::$generalConfigs['funnel']['currency'] = [
                'symbol' => static::$funnelJsonData->get('currency', '$'),
                'code'   => static::$funnelJsonData->get('currency_code', 'USD'),
        ];
    }

    private static function mapDomain() {
        $map = collect([
            "domain_name" =>"name",
            "copyright_name" =>"copyright_name",
            "address_inline" =>"address_inline",
            "address_formatted" =>"address_formatted",
            "address_return" =>"address_return",
            "server_ip" =>"server_ip",
            "notes" =>"notes",
        ]);

        foreach (static::$funnelJsonData as $key => $value) {

            if($map->has($key)) {
                static::$generalConfigs['domain'][$map->get($key)] = self::filterValue($value);
            }
        }
    }

    private static function mapSigner() {
        $map =  collect([
            "signer_corp_name" => "corp_name",
            "signer_first_name" => "first_name",
            "signer_last_name" => "last_name",
            "signer_phone" => "phone",
            "signer_email" => "email",
            "signer_status" => "status",
            "corporation" => "corporation",
            "state_of_incorporation" => "state_of_incorporation",
        ]);

        foreach (static::$funnelJsonData as $key => $value) {

            if($map->has($key)) {
                static::$generalConfigs['signer'][$map->get($key)] = self::filterValue($value);
            }
        }
    }

    public static function mapCompany() {
        $map = collect([
            "company_name" => "name",
            "company_first_name" => "first_name",
            "company_second_name" => "second_name",
            "company_email_address" => "email_address",
            "company_address" => "address",
            "company_return_address" => "return_address",
            "company_city" => "city",
            "company_state" => "state",
            "company_postal" => "postal",
            "company_phone" => "phone",
            "company_service_time" => "service_time",
        ]);

        foreach (static::$funnelJsonData as $key => $value) {

            if($map->has($key)) {
                static::$generalConfigs['company'][$map->get($key)] = self::filterValue($value);
            }
        }
    }

    public static function mapPaymentMethods() {
        
        static::$generalConfigs['payment_methods'] = [
            'credit_card' => [
                'types' => static::$funnelJsonData->get('cards', []),
                'test_cards' => static::$funnelJsonData->get('test_cards', []),
            ]
        ];
    }

    public static function mapEvents() {
        static::$generalConfigs['events'] = [
            'send_order_confirm_responder_email' => static::$funnelJsonData->get('order_confirm_responder', 0),
        ];
    }

    public static function mapPaymentProvider() {
        static::$generalConfigs['payment_provider'] = [
                "api_end_point" => static::$funnelJsonData->get('api_end_point', null),
                "api_type" => static::$funnelJsonData->get('api_type', null),
                "api_type_code" => static::$funnelJsonData->get('api_type_code', null),
                "api_user_name" => static::$funnelJsonData->get('api_user_name', null),
                "api_password" => static::$funnelJsonData->get('api_password', null),
                "api_guid" => static::$funnelJsonData->get('api_guid', null),
                "full_order_force_prospect" => static::$funnelJsonData->get('full_order_force_prospect', null),
                "m1_offer_id" => static::$funnelJsonData->get('m1_offer_id', null),
                "skip_website_url_partial" => static::$funnelJsonData->get('skip_website_url_partial', null),
                "is_geoplugin_api_enabled" => static::$funnelJsonData->get('is_geoplugin_api_enabled', null),
                'simulate_live_trans' => static::$funnelJsonData->get('simulate_live_trans', null),
                'll_3ds_gateway_id' => static::$funnelJsonData->get('ll_3ds_gateway_id', null)
        ];
    }
    
    public static function mapAddons() {

        static::$generalConfigs['addons'] = [
            'easy_cancellation' => [
                'is_order_cancellation_enabled' => false,
                'type' => 0,
            ],
            'state_tax' => static::$funnelJsonData->get('state_tax', null),
            'order_decline_config' => [
                'max_allowed_decline' => static::$funnelJsonData->get('max_allowed_decline', null),
                'session_ttl' => static::$funnelJsonData->get('order_block_time', null),
                'decline_msg' => static::$funnelJsonData->get('decline_msg', null),
            ],
            'smtp' => [
                'config_id' => static::$funnelJsonData->get('smtp_setting_id', null),
            ],
            'bin_blocks' => [
                "enable" => static::$funnelJsonData->get('is_block_bins', false),
                "bins" => static::$funnelJsonData->get('block_bins', null),
            ],
            'coupon' => [
                'codes' => static::$funnelJsonData->get('coupon_codes', null),
            ],
            'seo' => [
                'google_indexing' => !static::$funnelJsonData->get('disable_google_indexing'),
            ],
            'paay_threeDS' => [
                "enabled" => static::$funnelJsonData->get('is_paay3d_validation', false),
                "api_key" => static::$funnelJsonData->get('paay_api_key', null),
            ],
            '3d_validation' => [
                "enabled" => static::$funnelJsonData->get('is_3d_validation', false),
                "api_key" => static::$funnelJsonData->get('paay_api_key', null),
            ],
            'currency-converter' => [
                "enabled"=> static::$funnelJsonData->get('enable_currency_converter', false),
                "provider_id"=> static::$funnelJsonData->get('currency_converter.provider_id', null),
                "api_key" => static::$funnelJsonData->get('currency_converter_api_key', null),
            ],
            'script_management' => [
                'script' => static::$funnelJsonData->get('script_management', null),
            ],
            'custom_tokens' => static::$funnelJsonData->get('custom_tokens', []),
            // user_blocking_settings
            'time_based_user_blocking' => [
                "order_placing_interval_mins" => static::$funnelJsonData->get('blockTime', null),
                "block_type" => static::$funnelJsonData->get('blockType', []),
                "block_message" => static::$funnelJsonData->get('blockMessage', null),
            ]
        ];
    }
}
