<?php

namespace Framework\Modules;

//use Framework\Foundation\Config;
use Framework\Foundation\Helper;

class Pixel
{
    public $defultTimeZone      = 'UTC';
    public $pixelCheckingStatus = true;
    public $attrData            = [];
    public $affParams           = ['affid', 'afid', 'nid', 'sid', 'click_id', 'c1', 'c2', 'c3', 'c4', 'c5', 'c6'];
    public $headerTagPixel      = '';
    public $bodyStartTagPixel   = '';
    public $bodyEndTagPixel     = '';

    public function preparePixel($type)
    {
        $getConfigData = Helper::getJsonFileData(STORAGE_PATH, 'config.general.json');

        if (!empty($getConfigData['pixels']))
        {
            $getConfigData['pixels'] = $this->sortByPriority($getConfigData['pixels']);

            foreach ($getConfigData['pixels'] as $pixelInfo)
            {
                $this->pixelCheckingStatus = true;
                $this->attrData            = [];

                $pixelTxt = $this->pixelTriggers($pixelInfo, $type);
                $this->injectPixels($pixelInfo, $pixelTxt, $type);
            }
        }
        return [
            'header_tag_pixel'     => $this->headerTagPixel,
            'body_start_tag_pixel' => $this->bodyStartTagPixel,
            'body_end_tag_pixel'   => $this->bodyEndTagPixel
        ];
    }

    public function sortByPriority(array $pixels)
    {
        $priorityPrixel = [];
        $maxPriority    = max(array_column($pixels, 'priority'));
        $maxPriority    = empty($maxPriority) ? 0 : $maxPriority;
        foreach ($pixels as $pixelKey => $pixelValue)
        {
            if (empty($pixelValue['priority']))
            {
                $priorityPrixel[$pixelKey] = $maxPriority + 1;
            }
            else
            {
                $priorityPrixel[$pixelKey] = $pixelValue['priority'];
            }
        }
        array_multisort($priorityPrixel, SORT_ASC, $pixels);

        return $pixels;
    }

    public function checkingCustomSchedule(array $pixelInfo)
    {
        if ((!empty($pixelInfo['start_at'])) && (!empty($pixelInfo['stop_at'])))
        {
            $timezone = !empty($pixelInfo['timezone']) ? $pixelInfo['timezone'] : $this->defultTimeZone;
            $date     = new \DateTime("now", new \DateTimeZone($timezone));
            $currTime = strtotime($date->format('Y-m-d H:i:s'));

            if ($currTime >= strtotime($pixelInfo['start_at']) && $currTime <= strtotime($pixelInfo['stop_at']))
            {
                $this->pixelCheckingStatus = true;
            }
            else
            {
                $this->pixelCheckingStatus = false;
            }
        }
    }

    public function checkingDevice(array $pixelInfo)
    {
        if (($this->pixelCheckingStatus) && (!empty($pixelInfo['device'])))
        {
            $device         = json_decode($pixelInfo['device']);
            $tablet_browser = 0;
            $mobile_browser = 0;

            if (preg_match('/(tablet|ipad|playbook)|(android(?!.*(mobi|opera mini)))/i', strtolower($_SERVER['HTTP_USER_AGENT'])))
            {
                $tablet_browser++;
            }

            if (preg_match('/(up.browser|up.link|mmp|symbian|smartphone|midp|wap|phone|android|iemobile)/i', strtolower($_SERVER['HTTP_USER_AGENT'])))
            {
                $mobile_browser++;
            }

            if ((strpos(strtolower($_SERVER['HTTP_ACCEPT']), 'application/vnd.wap.xhtml+xml') > 0) or ((isset($_SERVER['HTTP_X_WAP_PROFILE']) or isset($_SERVER['HTTP_PROFILE']))))
            {
                $mobile_browser++;
            }

            $mobile_ua     = strtolower(substr($_SERVER['HTTP_USER_AGENT'], 0, 4));
            $mobile_agents = array('w3c ', 'acs-', 'alav', 'alca', 'amoi', 'audi', 'avan', 'benq', 'bird', 'blac', 'blaz', 'brew', 'cell', 'cldc', 'cmd-', 'dang', 'doco', 'eric', 'hipt', 'inno', 'ipaq', 'java', 'jigs', 'kddi', 'keji', 'leno', 'lg-c', 'lg-d', 'lg-g', 'lge-', 'maui', 'maxo', 'midp', 'mits', 'mmef', 'mobi', 'mot-', 'moto', 'mwbp', 'nec-', 'newt', 'noki', 'palm', 'pana', 'pant', 'phil', 'play', 'port', 'prox', 'qwap', 'sage', 'sams', 'sany', 'sch-', 'sec-', 'send', 'seri', 'sgh-', 'shar', 'sie-', 'siem', 'smal', 'smar', 'sony', 'sph-', 'symb', 't-mo', 'teli', 'tim-', 'tosh', 'tsm-', 'upg1', 'upsi', 'vk-v', 'voda', 'wap-', 'wapa', 'wapi', 'wapp', 'wapr', 'webc', 'winw', 'winw', 'xda ', 'xda-');

            if (in_array($mobile_ua, $mobile_agents))
            {
                $mobile_browser++;
            }

            if (strpos(strtolower($_SERVER['HTTP_USER_AGENT']), 'opera mini') > 0)
            {
                $mobile_browser++;
                //Check for tablets on opera mini alternative headers
                $stock_ua = strtolower(isset($_SERVER['HTTP_X_OPERAMINI_PHONE_UA']) ? $_SERVER['HTTP_X_OPERAMINI_PHONE_UA'] : (isset($_SERVER['HTTP_DEVICE_STOCK_UA']) ? $_SERVER['HTTP_DEVICE_STOCK_UA'] : ''));
                if (preg_match('/(tablet|ipad|playbook)|(android(?!.*mobile))/i', $stock_ua))
                {
                    $tablet_browser++;
                }
            }

            if ($tablet_browser > 0)
            {
                $useDevice = 'tablet';
            }
            else if ($mobile_browser > 0)
            {
                $useDevice = 'mobile';
            }
            else
            {
                $useDevice = 'desktop';
            }
            if (!in_array($useDevice, $device))
            {
                $this->pixelCheckingStatus = false;
            }
        }
    }

    public function checkAffiliateValue(array $pixelInfo)
    {
        $affiliateData = [];
        $affiliateData = !empty($pixelInfo['affiliate_type_1_info']) ? array_column($pixelInfo['affiliate_type_1_info'], 'value') : [];
        $affiliateKey  = strtolower($pixelInfo['affiliate_type_1']);

        if (($this->pixelCheckingStatus) && (!empty($affiliateData)))
        {
            $getAffiliateParams = parse_str($_SERVER['QUERY_STRING'], $affParam);
            $getAffiliateParams = array_change_key_case($affParam, CASE_LOWER);
            if (!empty($getAffiliateParams[$affiliateKey]) && in_array($getAffiliateParams[$affiliateKey], $affiliateData))
            {
                $this->pixelCheckingStatus = true;
            }
            else
            {
                $this->pixelCheckingStatus = false;
            }
        }
    }

    public function checkSubAffiliateValue(array $pixelInfo)
    {
        $subAffiliateData = !empty($pixelInfo['affiliate_type_2_info']) ? array_column($pixelInfo['affiliate_type_2_info'], 'value') : [];
        $subAffiliateKey  = strtolower($pixelInfo['affiliate_type_2']);

        if (($this->pixelCheckingStatus) && (!empty($subAffiliateData)))
        {
            $getSubAffiliateParams = parse_str($_SERVER['QUERY_STRING'], $subAffParam);
            $getSubAffiliateParams = array_change_key_case($subAffParam, CASE_LOWER);
            if (!empty($getSubAffiliateParams[$subAffiliateKey]) && in_array($getSubAffiliateParams[$subAffiliateKey], $subAffiliateData))
            {
                $this->pixelCheckingStatus = true;
            }
            else
            {
                $this->pixelCheckingStatus = false;
            }
        }
    }

    public function checkingPage(array $pixelInfo)
    {
        $currentPage = (isset($_SERVER['REDIRECT_URL'])) ? ltrim($_SERVER['REDIRECT_URL'], '/') : 'index';

        if (($this->pixelCheckingStatus) && (!empty($pixelInfo['page_inclusion'])))
        {
            if (!in_array($currentPage, json_decode($pixelInfo['page_inclusion'])))
            {
                $this->pixelCheckingStatus = false;
            }
        }

        if (($this->pixelCheckingStatus) && (!empty($pixelInfo['page_exclusion'])))
        {
            if (in_array($currentPage, json_decode($pixelInfo['page_exclusion'])))
            {
                $this->pixelCheckingStatus = false;
            }
        }
    }

    public function parseTokens($pixelTxt)
    {
        $attrData['order_id']    = !empty($_SESSION['twig_tokens']['order_id']) ? $_SESSION['twig_tokens']['order_id'] : '';
        $attrData['order_total'] = !empty($_SESSION['twig_tokens']['total_price']) ? $_SESSION['twig_tokens']['total_price'] : '';
        $getAffiliateParams      = parse_str($_SERVER['QUERY_STRING'], $affParam);
        $getAffiliateParams      = array_change_key_case($affParam, CASE_LOWER);

        foreach ($this->affParams as $affValue)
        {

            $attrData[$affValue] = (array_key_exists($affValue, $getAffiliateParams)) ? $getAffiliateParams[$affValue] : '';
        }
        $this->attrData = $attrData;

        return preg_replace_callback("/\{([a-z0-9_]+)\}/i", function ($pixelTxt)
        {
            foreach ($this->attrData as $key => $_value)
            {
                if (preg_match("/$pixelTxt[1]/i", $key))
                {
                    return $_value;
                }
            }
        }, $pixelTxt);
    }

    public function doPostBack($postbackUrl)
    {
        $ch     = curl_init();
        curl_setopt($ch, CURLOPT_URL, $postbackUrl);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_MAXREDIRS, 5);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 1);
        $output = curl_exec($ch);
        curl_close($ch);
        return $output;
    }

    public function checkingIsPrepaidOrder(array $pixel)
    {
        if (empty($pixel['on_prepaid']) && !empty($_SESSION['twig_tokens']['prepaid_match']))
        {
            $this->pixelCheckingStatus = false;
        }
    }

    public function checkingIsTestOrder(array $pixel)
    {
        if (empty($pixel['on_live']) && !empty($_SESSION['twig_tokens']['is_test_cc']))
        {
            $this->pixelCheckingStatus = false;
        }
    }

    public function checkingPixelType($pixel, $type)
    {
        $type = ['html', 'html_postback', 'postback', 'decline'];

        if (in_array($pixel['type'], $type) && empty($_SESSION['order_details']))
        {
            $this->pixelCheckingStatus = false;
        }
        if (($pixel['type'] == 'decline') && empty($_SESSION['twig_tokens']['is_decline']))
        {
            $this->pixelCheckingStatus = false;
        }
        if (($pixel['type'] == 'form_submit') && ($type == 'general'))
        {
            $this->pixelCheckingStatus = false;
        }
    }

    public function pixelTriggers($pixel, $type)
    {
        $this->checkingCustomSchedule($pixel);
        $this->checkingDevice($pixel);
        $this->checkAffiliateValue($pixel);
        $this->checkSubAffiliateValue($pixel);
        $this->checkingPage($pixel);
        $this->checkingIsPrepaidOrder($pixel);
        $this->checkingIsTestOrder($pixel);
        $this->checkingPixelType($pixel, $type);

        $pixeltxt = '';
        if ($this->pixelCheckingStatus)
        {
            if ($pixel['type'] == 'html')
            {
                $pixeltxt = $this->parseTokens($pixel['pixel']);
            }
            else if ($pixel['type'] == 'html_postback')
            {
                $url = $this->parseTokens($pixel['postback_url']);
                $this->doPostBack($url);
            }
            else if ($pixel['type'] == 'postback')
            {
                if (!empty($pixel['postback_url']))
                {
                    $url = $this->parseTokens($pixel['postback_url']);
                    $this->doPostBack($url);
                }

                if (!empty($pixel['pixel']))
                {
                    $pixeltxt = $this->parseTokens($pixel['pixel']);
                }
            }
            else
            {
                $pixeltxt = $this->parseTokens($pixel['type']);
            }
        }
        return $pixeltxt;
    }

    public function injectPixels($pixelInfo, $pixel)
    {
        if ($this->pixelCheckingStatus)
        {
            if ($pixelInfo['placement'] == 'head')
            {
                $this->headerTagPixel = $this->headerTagPixel . PHP_EOL . $pixel;
            }
            if ($pixelInfo['placement'] == 'body_open')
            {
                $this->bodyStartTagPixel = $this->bodyStartTagPixel . PHP_EOL . $pixel;
            }
            if ($pixelInfo['placement'] == 'body_close')
            {
                $this->bodyEndTagPixel = $this->bodyEndTagPixel . PHP_EOL . $pixel;
            }
        }
    }

}
