<?php

if(!empty($_COOKIE['UNIFY_DEBUG_MODE']) && $_COOKIE['UNIFY_DEBUG_MODE'] === 'ENABLE_DEBUGGER') {
    $appDebug = true;
} else {
    $appDebug = false;
}

defined('APP_DEBUG') ? null : define('APP_DEBUG', $appDebug);
defined('APP_NAME') ? null : define('APP_NAME', 'unify-cms-v3');
defined('APP_ENV') ? null : define('APP_ENV', 'local');
defined('APP_SESSION_ROOT_KEY') ? null : define('APP_SESSION_ROOT_KEY', APP_NAME);
defined('PHP_VERSION') ? null : define('PHP_VERSION', phpversion());
defined('CMS_BAREBONE_VERSION') ? null : define('CMS_BAREBONE_VERSION', '4.0.85');
defined('BASE_PATH') ? null : define('BASE_PATH', dirname(__DIR__));
defined('DS') ? null : define('DS', DIRECTORY_SEPARATOR);
defined('STORAGE_PATH') ? null : define('STORAGE_PATH', BASE_PATH . DS . 'storage');
defined('CUSTOM_STORAGE_PATH') ? null : define('CUSTOM_STORAGE_PATH', BASE_PATH . DS . 'custom_storage');
defined('UNI_APP_DB_CONN') ? null : define('UNI_APP_DB_CONN', 'sqlite_connection');

// Error handling codes
defined('AJAX_EX_CODE') ? null : define('AJAX_EX_CODE', 1000);

/**
 * If sets to TRUE and CMS_BAREBONE_VERSION will be updated to new version,
 * it will drop all existing tables in datbase with records and can not be revert back.
 */
defined('FORCE_DB_MIGRATION') ? null : define('FORCE_DB_MIGRATION', false);

if (APP_DEBUG)
{
    error_reporting(E_ALL);
}
ini_set('display_errors', APP_DEBUG ? 1 : 0);

if (PHP_VERSION < 5.6 || PHP_VERSION > 8)
{
    die("<html>
        <head>
                <title>PHP Version Incompatible</title>
        </head>
        <body style='padding:10vw; text-align:center; color:#777; font-weight:bold; font-size:200%;'>
            The current version of PHP that you are using is: ".PHP_VERSION." <br> 
            Our system supports PHP versions that are greater than 5.6 and less than 8.0.
        </body>
    </html>");
}

if (!function_exists('app_configs'))
{

    /**
     * Read app.config.php configs based on current APP_ENV
     * @param string @env
     * @return Array
     */
    function app_configs($env = null)
    {
        $configs = require BASE_PATH . DS . 'Framework' . DS . 'app.config.php';
        $currEnv = !empty($env) ? $env : APP_ENV;

        if (in_array($currEnv, ['local', 'development']))
        {
            return $configs['development'];
        }

        return $configs['production'];
    }

}

if (!function_exists('app'))
{

    /**
     * Create a singleton application instacne
     * @return \Freamework\Foundation\Application
     */
    function app()
    {

        return \Framework\Foundation\Application::getInstance();
    }

}

if (!function_exists('pre_response_array'))
{

    /**
     *
     * @param string $message
     * @param bool $success
     * @param array $result
     * @param array $resultInfo
     * @param array|object $errors
     * @param type $ex
     * @return array
     */
    function pre_response_array($message = NULL, $success = true, $result = NULL, $resultInfo = NULL, $errors = NULL, $ex = NULL)
    {
        return [
            'result'       => $result,
            'apiResponse'  => $result,
            'data'         => $result,
            'status'       => $success ? 'success' : 'error',
            'success'      => $success,
            'errors'       => $errors,
            'message'      => $message,
            'result_info'  => $resultInfo,
            'ex_message'   => $message,
            'dev_response' => ((APP_DEBUG && (!empty($ex))) ? exception_formatter($ex) : null)
        ];
    }

}

if (!function_exists('exception_formatter'))
{

    /**
     * @param Throwable|Exception $ex
     * @return array
     */
    function exception_formatter($ex)
    {
        return [
            'message' => $ex->getMessage(),
            'file'    => $ex->getFile(),
            'line'    => $ex->getLine(),
        ];
    }

}

if (!function_exists('appLog'))
{

    /**
     * @return void
     */
    function appLog($operation, $success, $msg)
    {
        $ip = \Framework\Foundation\Helper::getIP();
        \Framework\Modules\UserAccess::log($ip, $operation, $success, $msg);
    }

}

