<?php

/**
 * @version 1.0.1
 * Install Script to extract modules
 */
error_reporting(0);
@ini_set('display_errors', 0);
set_time_limit(3600);


$requestURI = array_filter(explode('/', $_SERVER['PATH_INFO']));
$scriptName = array_filter(explode('/', $_SERVER['SCRIPT_NAME']));
parse_str($_SERVER['QUERY_STRING'], $getVariables);

for ($i = 1; $i < sizeof($scriptName); $i++)
{
    if ($requestURI[$i] == $scriptName[$i])
    {
        unset($requestURI[$i]);
    }
}

$command = array_values($requestURI);

if (empty($_GET['route']) && !count($command))
{
    template();
}
else
{
    $route = empty($_GET['route']) ? array_shift($command) : $_GET['route'];
    //$route = $_GET['route'];
    switch ($route)
    {
        case 'prerequisite':
            prerequisite();
            break;
        case 'extract':
            extractContent();
            break;
        case 'clear':
            clearContent();
            break;
        case 'archive':
            makeArchive();
            break;
        case 'download':
            downloadFile();
            break;
        case 'renamengApp':
            renamengApp();
            break;
        case 'analyzeTheme':
            analyzeTheme();
            break;
        case 'zipBall':
            zipBall();
            break;
        case 'allZipBall':
            allZipBall();
            break;
        case 'renameFunnelPath':
            renameFunnelPath();
            break;
        case 'extractSingle':
            extractSingleContent();
            break;
        case 'moveSingle':
             moveSingle();
            break;
        case 'instant_download':
            instantDownload();
            break; 
        case 'extractAddon':
            extractAddon();
            break;
        case 'checkCopiedFunnelStatus' : 
            checkCopiedFunnelStatus();
            break;
        case  'renameFilesDirectories' : 
            renameFilesDirectories();
            break;
        case 'deleteFileDirectories' : 
            deleteFileDirectories();
            break;
        case 'convertOffline' : 
            convertOffline();
            break;
        case 'videoextract':
            videoExtractContent();
            break;    
        default:
            header("HTTP/1.0 404 Not Found");
            json([]);
    }
    die();
}

/**
 * Check the server configuration
 */
function prerequisite()
{
    $type = $_GET['type'];
    if (!in_array($type, ['sales', 'compliant']))
    {
        return json(array(
            "success"   => false,
            "userInfo"  => getWebServerUserInfo(),
            "errorType" => 'argMissing',
            "errors"    => ["Type argument is invalid"]
        ));
    }

    require_once 'pclzip.lib.php';

    $errors = array();
    if (!is_writable(__DIR__))
    {
        array_push($errors, "Project directory does\'t have writable permission");
    }

    if (!is_executable(__DIR__))
    {
        array_push($errors, "Project directory does\'t have executable permission.");
    }

    if (count($errors))
    {
        return json(array(
            "success"   => false,
            "userInfo"  => getWebServerUserInfo(),
            "errorType" => 'permission',
            "errors"    => $errors
        ));
    }

    $complianceDirectory = array('css', 'Framework', 'frontend', 'images', 'storage', 'templates', 'vendor');
    $salesDirectory      = array('admin', 'app', 'assets', 'langs', 'library', 'member', 'mobile', 'tmp', 'upsellmanage');


    $duePermission    = array();
    $iterativeDirList = $type === 'sales' ? $salesDirectory : $complianceDirectory;
    foreach ($iterativeDirList as $index => $directory)
    {
        $d = sprintf("%s%s%s", __DIR__, DIRECTORY_SEPARATOR, $directory);
        if (is_dir($d) && !is_writable($d))
        {
            array_push($duePermission, $directory);
        }
    }

    if ($duePermission)
    {
        return json(array(
            "success"   => false,
            "userInfo"  => getWebServerUserInfo(),
            "errorType" => 'permission',
            "errors"    => [sprintf("%s %s does\'t have writable permission.",
                        implode(',', $duePermission),
                        count($duePermission) > 1 ? 'directories' : 'directory'
                )]
        ));
    }


    $extractOptions = 0;
    $fileName       = __DIR__ . DIRECTORY_SEPARATOR . 'installation.zip';
    $directoryName  = __DIR__ . DIRECTORY_SEPARATOR . 'prerequisite';
    if (is_dir($directoryName))
        rmdir($directoryName);
    mkdir($directoryName);

    ob_start();
    if (!$extractOptions && function_exists('exec') && exec('command -v unzip'))
    {
        exec("unzip $fileName $directoryName");
        if (checkDir($directoryName))
            $extractOptions = 1;
    }

    if (!$extractOptions && function_exists('system') && system('command -v unzip'))
    {
        system("unzip $fileName $directoryName");
        if (checkDir($directoryName))
            $extractOptions = 2;
    }

    if (!$extractOptions && class_exists('ZipArchive'))
    {
        $zip = new ZipArchive;

        if ($zip->open($fileName) === TRUE)
        {
            $zip->extractTo($directoryName);
            $zip->close();
            if (checkDir($directoryName))
                $extractOptions = 3;
        }
    }

    if (!$extractOptions && class_exists('PclZip'))
    {
        $archive = new PclZip($fileName);
        if ($archive->extract(PCLZIP_OPT_PATH, $directoryName) != 0)
        {
            if (checkDir($directoryName))
                $extractOptions = 4;
        }
    }

    $result = ob_get_contents();
    ob_end_clean();

    return json(array(
        "success" => true,
        "method"  => $extractOptions
    ));
}

/**
 * Extract Content
 */
function extractContent()
{
    $type = trim($_GET['type']);
    if (!in_array($type, ['sales', 'compliant']))
    {
        return json(array(
            "success"   => false,
            "errorType" => 'argMissing',
            "errors"    => ["Type argument is invalid."]
        ));
    }

    $filesName = trim($_GET['fileName']);
    if (empty($filesName))
    {
        return json(array(
            "success"   => false,
            "errorType" => 'fileNameMissing',
            "errors"    => ["File name argument is invalid."]
        ));
    }

    $method = trim($_GET['method']);
    if (empty($method) || !is_numeric($method))
    {
        return json(array(
            "success"   => false,
            "errorType" => 'extractTypeMissing',
            "errors"    => ["Extraction type argument is invalid."]
        ));
    }

    $doNotRemoveInstallScript = $_GET['remove'] == true ? true : false;

    $skipFiles = trim($_GET['skipFiles']);

    $errors = [];
    foreach (explode(',', $filesName) as $toExtractFileName)
    {
        $fileName      = sprintf("%s%s%s.zip", __DIR__, DIRECTORY_SEPARATOR, $toExtractFileName);
        $directoryName = __DIR__;
        file_put_contents(dirname(__FILE__).'/extract-error.php', "234 line".sprintf("%s.zip", $fileName)."\r\n",FILE_APPEND);
        try
        {
            switch ($method)
            {
                case 1: exec("unzip $fileName $directoryName");
                    break;
                case 2: system("unzip $fileName $directoryName");
                    break;
                    case 3: 
                        useZipArchive($fileName, $directoryName);
                        $extractedPath = getExtractedPath(__DIR__, $toExtractFileName);
                        try{
                            if(
                                is_dir($extractedPath) 
                                && (preg_match("/^unify-to-framework-barebone/i",$extractedPath) || preg_match("/^unify-cms-framework-barebone-v5/i",$extractedPath))
                                ){
                                updateNewKeysToStorage($extractedPath);
                            }
                        }catch (Exception $ex){
                            
                        }
                        if ($skipFiles)
                        {
                            skipFiles(
                                    sprintf("%s%s%s", __DIR__, DIRECTORY_SEPARATOR, $extractedPath),
                                    explode(',', $skipFiles)
                            );
                        }
                        move(sprintf("%s%s%s", __DIR__, DIRECTORY_SEPARATOR, $extractedPath), $directoryName);
                        break;
                    case 4: usePCLZip($fileName, $directoryName);
                        $extractedPath = getExtractedPath(__DIR__, $toExtractFileName);
                        try{
                            if(
                                is_dir($extractedPath) 
                                && (preg_match("/^unify-to-framework-barebone/i",$extractedPath) || preg_match("/^unify-cms-framework-barebone-v5/i",$extractedPath))
                            ){
                                updateNewKeysToStorage($extractedPath);
                            }
                        }catch (Exception $ex){
                            
                        }
                        if ($skipFiles){
                            skipFiles(sprintf("%s%s%s", __DIR__, DIRECTORY_SEPARATOR, $extractedPath), explode(',', $skipFiles));
                        }
                        move(sprintf("%s%s%s", __DIR__, DIRECTORY_SEPARATOR, $extractedPath), $directoryName);
                        break;
                case 5:
                    try{
                        if(!empty($_GET['removeDirectories'])){
                            foreach(explode(',',$_GET['removeDirectories']) as $dir){
                                $d = sprintf("%s%s%s", __DIR__, DIRECTORY_SEPARATOR, $dir);
                                if (is_dir($d))
                                {
                                    deleteDirectory($d);
                                }
                            }
                        }
                    }catch(Exception $ex){
                        file_put_contents(dirname(__FILE__).'/extract-error.php', "Case 5 catch -".$ex->getMessage()."\r\n",FILE_APPEND);
                    }
                    usePCLZip($fileName, $directoryName);
                    break;
                default: 
                array_push($errors, 'no specific exract method found');
                break;
            }

            /**
             * Add Custom Htaccess
             */
            if($type == "compliant")
                addCustomHtaccess();
        }
        catch (Exception $ex)
        {
            file_put_contents(dirname(__FILE__).'/extract-error.php', "275 line".$ex->getMessage()."\r\n",FILE_APPEND);
            $success = false;
            array_push($errors, $ex->getMessage());
        } finally
        {
            file_put_contents(dirname(__FILE__).'/extract-error.php', "280 line".sprintf("%s.zip", $toExtractFileName)."\r\n",FILE_APPEND);
            try{
                @unlink(sprintf("%s.zip", $toExtractFileName));
            }catch(Exception $unlinkex){
                file_put_contents(dirname(__FILE__).'/extract-error.php', "284 line".$unlinkex->getMessage()."\r\n",FILE_APPEND);
            }
        }
    }
    file_put_contents(dirname(__FILE__).'/extract-error.php', "288"."\r\n",FILE_APPEND);

    if (!$doNotRemoveInstallScript)
        removeInstallationScripts();

    if (count($errors))
    {
        return json(array(
            "success"   => false,
            "errorType" => 'extractError',
            "errors"    => $errors
        ));
    }

    // Do Funnel related staff
    return json(array(
        "success" => true,
        "message" => $_GET
    ));
}

/**
 * Clear Install Script
 */
function clearContent()
{

    $message = "Install Script cleared.";

    $filesName = trim($_GET[filesName]);

    try
    {
        removeInstallationScripts(explode(',', $fileName));
    }
    catch (Exception $err)
    {
        $message = $err->getMessage();
    }

    return json(array(
        "success" => true,
        "message" => $message
    ));
}

/**
 * Make archive
 */
function makeArchive()
{
    $fileName = trim($_GET['fileName']);
    if (empty($fileName))
    {
        return json(array(
            "success"   => false,
            "errorType" => 'fileNameMissing',
            "errors"    => ["File name argument is invalid."]
        ));
    }

    $method = trim($_GET['method']);
    if (empty($method) || !is_numeric($method))
    {
        return json(array(
            "success"   => false,
            "errorType" => 'extractTypeMissing',
            "errors"    => ["Extraction type argument is invalid."]
        ));
    }

    $errors       = [];
    $prerequisite = array('install.php', 'pclzip.lib.php', 'installation.zip', 'modifiedZip.lib.php');

    try
    {
        switch ($method)
        {
            case 1: exec(sprintf("zip -r %s . %s", $fileName, implode(' ', array_map('xCluderCmd', $prerequisite))));
                break;
            case 2: system(sprintf("zip -r %s . %s", $fileName, implode(' ', array_map('xCluderCmd', $prerequisite))));
                break;
            case 3: makeArchiveUsingZipArchive($fileName, $prerequisite);
                break;
            default: break;
        }
    }
    catch (Exception $ex)
    {
        $success = false;
        array_push($errors, $ex->getMessage());
    }

    if (count($errors))
    {
        return json(array(
            "success"   => false,
            "errorType" => 'extractError',
            "errors"    => $errors
        ));
    }

    return json(array(
        "success" => true,
        "message" => $_GET
    ));
}

/**
 * Download zip archive
 */
function downloadFile()
{
    $fileName = trim($_GET['fileName']);
    if (empty($fileName))
    {
        return json(array(
            "success"   => false,
            "errorType" => 'fileNameMissing',
            "errors"    => ["File name argument is invalid."]
        ));
    }

    if (!file_exists(sprintf("%s%s%s", __DIR__, DIRECTORY_SEPARATOR, $fileName)))
    {
        return json(array(
            "success"   => false,
            "errorType" => 'fileNotExists',
            "errors"    => ["File does not exist."]
        ));
    }

    ob_get_clean();
    header("Pragma: public");
    header("Expires: 0");
    header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
    header("Cache-Control: private", false);
    header("Content-Type: application/zip");
    header("Content-Disposition: attachment; filename=" . basename($fileName) . ";");
    header("Content-Transfer-Encoding: binary");
    header("Content-Length: " . filesize($fileName));
    readfile_chunked($fileName);
}

/**
 * Index Content
 */
function template()
{
    return json(array(
        "status" => "true"
    ));
}

/**
 * JSON Reponse
 */
function json($content)
{
    header("Content-Type: application/json;charset=utf-8");
    echo  json_encode($content);
   // header('Content-Type: application/json');
    //print json_encode($content);
    exit();
}

/**
 * Check Directory Exist
 */
function checkDir($directoryName)
{
    $files = array_diff(scandir($directoryName), [".", ".."]);
    deleteDirectory($directoryName);
    if ($files && in_array('test.php', $files))
        return true;
    return false;
}

/**
 * Skip file to move
 */
function skipFiles($path, $files = [])
{
    if (!is_array($files))
        return;

    foreach ($files as $file)
    {
        if($file == 'htaccess'){
            $file = '.htaccess';
        }
        $finalPath = sprintf("%s%s%s", $path, DIRECTORY_SEPARATOR, $file);

        if (!file_exists($finalPath))
            continue;
        if (is_dir($finalPath))
            deleteDirectory($finalPath);
        else
            @unlink($finalPath);
    }
}

/**
 * Delete not empty directory
 */
function deleteDirectory($dir)
{
    if (!file_exists($dir))
    {
        return true;
    }

    if (!is_dir($dir))
    {
        return unlink($dir);
    }

    foreach (scandir($dir) as $item)
    {
        if ($item == '.' || $item == '..')
        {
            continue;
        }

        if (!deleteDirectory($dir . DIRECTORY_SEPARATOR . $item))
        {
            return false;
        }
    }

    return rmdir($dir);
}

 function updateNewKeysToStorage($barebonePath){
        $path = 'storage'.DIRECTORY_SEPARATOR.'admin'.DIRECTORY_SEPARATOR;
        $barebonePath = $barebonePath.DIRECTORY_SEPARATOR.$path;
        
        $results = array_diff(scandir($path, 1), [".", ".."]);
        if(empty($results))
            return false;
        $pattern = "/.*\.(json)$/i";
        $configPattern = '/.*\.config\.(json)$/i';
        $dataPattern = '/.*\.data\.(json)$/i';
        foreach ($results as $key => $storageFile) {
            if(!preg_match($configPattern,$storageFile))
                continue;
                $existingFileContent = @file_get_contents($path.str_replace('.config', '.data', $storageFile));
                $bareboneContent = @file_get_contents($barebonePath.$storageFile);
                if(empty($bareboneContent))
                    continue;
                $bareboneDataKeys = json_decode($bareboneContent, true);
                if(empty($bareboneDataKeys) || empty($bareboneDataKeys['schema'])){
                    continue;
                }
                if ($existingFileContent) {
                    $currentDataKeys = json_decode($existingFileContent, true);
                    foreach($currentDataKeys as $k => $oldKeys)
                    {
                        if(count($bareboneDataKeys['schema']) != count($oldKeys))
                        {
                            $extraKeys = array_diff_key($bareboneDataKeys['schema'],$oldKeys);
                            array_walk_recursive(
                                $extraKeys,
                                function (&$v) {
                                    $v = assignValByDataType($v);                                    
                                }
                            );
                            $oldKeys = array_merge_recursive($oldKeys, $extraKeys);
                        }
                        array_walk_recursive(
                            $oldKeys,
                            function (&$v) {
                                $v = is_null($v)? "" : $v;                                    
                            }
                        );
                        $currentDataKeys[$k] = $oldKeys;
                    }
                }
                if(!empty($currentDataKeys)){
                    file_put_contents($path.str_replace('.config', '.data', $storageFile), 
                            json_encode($currentDataKeys), LOCK_EX
                    );
                }
                $existingConfigFileContent = @file_get_contents($path.$storageFile);
                if($existingConfigFileContent){
                    $existingConfigFileData = json_decode($existingConfigFileContent, true);
                }
                if(!empty($existingConfigFileData)){
                    $existingConfigFileData['schema'] = $bareboneDataKeys['schema'];
                    file_put_contents($path.$storageFile, json_encode($existingConfigFileData), LOCK_EX);
                }
        }

    }
    
    function assignValByDataType($type)
    {
        switch ($type)
        {
            case 'boolean':
                $val = false;
                break;
            case 'integer':
                $val = 0;
                break;
            default :
                $val = "";
                break;
        }
        return $val;
    }

/**
 * Update Google Indexing
 */
function updateGoogleIndexing($shouldDisableGoogleIndexing = true)
{

    try
    {
        $robotTxtFile = sprintf("%s%s%s", __DIR__, DIRECTORY_SEPARATOR, 'robots.txt');
        $robotStr     = is_readable($robotTxtFile) ? @file_get_contents($robotTxtFile) : "";

        $find = "User-agent:*Disallow:/";
        if (!strlen(trim($robotStr)))
        {
            $isMatched = true;
        }
        else
        {
            $isMatched = findString($robotStr, $find);
        }

        if (!$shouldDisableGoogleIndexing && $isMatched)
        {
            @file_put_contents($robotTxtFile, "User-agent: *\nAllow: /");
        }
    }
    catch (Exception $e)
    {
        // ignore exception
    }
}

/**
 * Get User Information
 */
function getWebServerUserInfo()
{
    $data = ['www_user' => 'N/A'];
    if (function_exists('posix_getpwuid'))
    {
        $userinfo         = posix_getpwuid(posix_getuid());
        if (!empty($userinfo['name']))
            $data['www_user'] = $userinfo['name'];
        if (!empty($userinfo['uid']))
            $data['www_user'] .= '(' . $userinfo['uid'] . ')';
    }

    return $data;
}

/**
 * Extract using ZipArchive Methods
 */
function useZipArchive($fileName, $destination)
{
    if (!class_exists('ZipArchive'))
        throw new Exception("ZipArchive Method not found.");
    if (empty($fileName))
        throw new Exception("File name is invalid.");
    if (empty($destination))
        throw new Exception("Destination is invalid.");

    $zip = new ZipArchive;
    $res = $zip->open($fileName);
    file_put_contents(dirname(__FILE__).'/extract-error.php', "Response".$res."\r\n",FILE_APPEND);
    if ($res === TRUE)
    {
        if($zip->extractTo($destination)){
            file_put_contents(dirname(__FILE__).'/extract-error.php', "destination".$destination."\r\n",FILE_APPEND);
        }
        $zip->close();

        return true;
    }
    file_put_contents(dirname(__FILE__).'/extract-error.php', "590".$res."\r\n",FILE_APPEND);
    $ErrMsg = null;
    switch($res){
        case ZipArchive::ER_EXISTS:
            $ErrMsg = "File already exists.";
            break;

        case ZipArchive::ER_INCONS:
            $ErrMsg = "Zip archive inconsistent.";
            break;
           
        case ZipArchive::ER_MEMORY:
            $ErrMsg = "Malloc failure.";
            break;
           
        case ZipArchive::ER_NOENT:
            $ErrMsg = "No such file.";
            break;
           
        case ZipArchive::ER_NOZIP:
            $ErrMsg = "Not a zip archive.";
            break;
           
        case ZipArchive::ER_OPEN:
            $ErrMsg = "Can't open file.";
            break;
           
        case ZipArchive::ER_READ:
            $ErrMsg = "Read error.";
            break;
           
        case ZipArchive::ER_SEEK:
            $ErrMsg = "Seek error.";
            break;
       
        default:
            $ErrMsg = "Unknow (Code $rOpen)";
            break;
           
       
    }  
    file_put_contents(dirname(__FILE__).'/extract-error.php', $ErrMsg."\r\n",FILE_APPEND);
    throw new Exception($ErrMsg);
}

/**
 * Extract File using PCLZip Library
 */
function usePCLZip($fileName, $destination)
{
    if (empty($fileName))
        throw new Exception("File name is invalid.");
    if (empty($destination))
        throw new Exception("Destination is invalid.");

    require_once 'pclzip.lib.php';
    if (!class_exists('PclZip'))
        throw new Exception("PclZip Method not found.");


    $archive = new PclZip($fileName);
    if ($archive->extract(PCLZIP_OPT_PATH, $destination) == 0)
    {
        throw new Exception($archive->errorInfo(true));
    }

    return true;
}

/**
 * Remove installation Scripts
 */
function removeInstallationScripts($files = array())
{
    $prerequisite = array('install.php', 'pclzip.lib.php', 'installation.zip', 'modifiedZip.lib.php');
    if (is_array($files))
        $prerequisite = array_merge($prerequisite, $files);

    foreach ($prerequisite as $index => $file)
    {
        $file = sprintf("%s%s%s", __DIR__, DIRECTORY_SEPARATOR, $file);
        if (file_exists($file))
            @unlink($file);
    }
}

/**
 * Move Files
 */
function move($source, $destination, $type = 0)
{
    if (empty($source))
        throw new Exception("Source is invalid.");
    if (empty($destination))
        throw new Exception("Destination is invalid.");

    if ($type)
    {
        ob_start();
        $moveCommad = "cd $source && find . -maxdepth 1 -exec mv {} .. \\";
        if ($type == 1)
            exec($moveCommad);
        elseif ($type == 2)
            exec($moveCommad);

        $output = ob_get_contents();
        ob_end_clean();
        return;
    }

    rcopy($source, $destination);
    deleteDirectory($source);
}

/**
 * Recursive copy
 */
function rcopy($src, $dst)
{
    try {
        if(`which cp`) {
            $src = rtrim($src, '/');
            $dst = rtrim($dst, '/');
            @mkdir($dst);
            `cp -fr {$src}/. {$dst}/`;
        } else {
            throw new Exception("Mv failed");
        }
    } catch (Exception $e) {
        $dir  = opendir($src);
        @mkdir($dst);
        while (false !== ( $file = readdir($dir)))
        {
            if (( $file != '.' ) && ( $file != '..' ))
            {
                if (is_dir($src . '/' . $file))
                {
                    rcopy($src . '/' . $file, $dst . '/' . $file);
                }
                else
                {
                    copy($src . '/' . $file, $dst . '/' . $file);
                }
            }
        }
        closedir($dir);
    }
}

/**
 * Get Extracted Path
 */
function getExtractedPath($directory, $fileName)
{
    $files = array_diff(scandir($directory), [".", ".."]);

    $path = null;
    foreach ($files as $index => $file)
    {
        if (startsWith($file, sprintf("%s-", $fileName)))
        {
            $path = $file;
        }
    }

    if (is_null($path))
        throw new Exception("Path not found.", 1);
    return $path;
}

/**
 * Find Start with
 */
function startsWith($haystack, $needle)
{
    $length = strlen($needle);
    return substr($haystack, 0, $length) === $needle;
}

/**
 * Exlude Command prepare
 */
function xCluderCmd($each)
{
    return sprintf("-x '*%s*'", $each);
}

/**
 * Make archive using Zip Archive
 */
function makeArchiveUsingZipArchive($fileName, $excludeFiles = array(), $skipExtension = array())
{
    if (empty($fileName))
        throw new Exception("File name is invalid.");
    if (!is_array($excludeFiles))
        throw new Exception("Exclude files is invalid");

    require_once 'modifiedZip.lib.php';

    $za = new ModifiedFlxZipArchive;

    //create an archive
    if ($za->open($fileName, ZipArchive::CREATE))
    {

        $za->addDirDoo(__DIR__, $fileName, $excludeFiles, $skipExtension);
        $za->close();
    }
    else
    {
        throw new Exception("Unable to create archive.");
    }
}

/**
 * Rename ANgular App Name for funnel
 */
function renamengApp()
{
    $guid = trim($_GET['guid']);
    if (empty($guid))
    {
        return json(array(
            "success"   => false,
            "errorType" => 'guidMissing',
            "errors"    => ["guid is invalid."]
        ));
    }
    $toReplace        = !empty($_GET['replace']) ? $_GET['replace'] : [];
    $knownModule      = array_merge(['ElminaraApp', '5edf407dd0e6'], $toReplace);
    $knownPath        = ['./templates'];
    $knownFiles       = ['./frontend/js/app.js'];
    $monitorExtension = ['html', 'htm', 'js'];
    $files            = [];

    foreach ($knownPath as $fileDir)
    {
        $files = getfileListing($fileDir, $files, $monitorExtension);
    }

    $allFiles = array_merge($files, $knownFiles);
    $files    = replaceStrFromFile($guid, $knownModule, $allFiles);
    return json(array(
        "success" => true,
        "message" => 'Rename App Sucessfully'
    ));
}

/**
 * Get file listing from directory
 * @param string $knownPath which dir want to be replace
 * @param string $results file listing array
 * @param string $monitorExtension which extension want to be replace
 */
function getfileListing($knownPath, &$results = array(), $monitorExtension)
{
    $files = scandir($knownPath);
    if (!empty($files))
    {
        foreach ($files as $file)
        {
            $path = $knownPath . DIRECTORY_SEPARATOR . $file;

            if (!is_dir($path))
            {
                $ext = pathinfo($path, PATHINFO_EXTENSION);

                if (in_array($ext, $monitorExtension))
                {
                    $results[] = $path;
                }
            }
            else
            {

                if ($file != "." && $file != "..")
                {
                    getfileListing($path, $results, $monitorExtension);
                }
            }
        }
    }

    return $results;
}

/**
 * Replace string from file
 * @param string $guid replace string
 * @param string $knownModule which string want to be replace
 * @param array $allFiles file listing array
 */
function replaceStrFromFile($guid, $knownModule, $allFiles)
{
    if (!empty($allFiles))
    {
        foreach ($allFiles as $file)
        {
            $getContent = file_get_contents($file);
            $str        = str_replace($knownModule, $guid, $getContent);
            file_put_contents($file, $str);
        }
    }
}

/**
 * Detect theme from existing offer
 * this method is used for migration
 * funnel task.
 */
function analyzeTheme()
{
    $fileName = trim($_GET['fileName']);
    if (empty($fileName))
    {
        return json(array(
            "success"   => false,
            "errorType" => 'fileNameMissing',
            "errors"    => ["File name argument is invalid."]
        ));
    }

    $errors         = [];
    $skipForArchive = array(
        'install.php',
        'pclzip.lib.php',
        'installation.zip',
        'modifiedZip.lib.php',
        'admin',
        'ajax.php',
        'api.php',
        'assets',
        'extensions',
        'langs',
        'library',
        'phpliteadmin.php',
        'storage',
        'tmp',
        'vendor',
        '.htaccess',
        'error_log',
        '.git'
    );

    if (!empty($_GET['excludeFiles']) && is_array($_GET['excludeFiles']))
    {
        $skipForArchive = array_merge($skipForArchive, $_GET['excludeFiles']);
    }


    try
    {
        makeArchiveUsingZipArchive($fileName, $skipForArchive, ['zip']);
    }
    catch (Exception $ex)
    {
        $success = false;
        array_push($errors, $ex->getMessage());
    }

    if (count($errors))
    {
        return json(array(
            "success"   => false,
            "errorType" => 'themeError',
            "errors"    => $errors
        ));
    }

    return json(array(
        "success" => true,
        "message" => $_GET
    ));
}

/**
 * Read File By Chunked and steam for download.
 */
function readfile_chunked($filename, $retbytes = TRUE)
{
    define('CHUNK_SIZE', 1024 * 1024);
    $buffer = '';
    $cnt    = 0;
    $handle = fopen($filename, 'rb');

    if ($handle === false)
    {
        return false;
    }

    while (!feof($handle))
    {
        $buffer = fread($handle, CHUNK_SIZE);
        echo $buffer;
        ob_flush();
        flush();

        if ($retbytes)
        {
            $cnt += strlen($buffer);
        }
    }

    $status = fclose($handle);

    if ($retbytes && $status)
    {
        return $cnt; // return num. bytes delivered like readfile() does.
    }

    return $status;
}

/**
 * Create ZipBall with given file list
 */
function zipBall()
{

    $fileName    = explode(",", trim($_GET['paths']));
    $archiveName = trim($_GET['archive']);

    if (empty($fileName))
    {
        return json(array(
            "success"   => false,
            "errorType" => 'fileNameMissing',
            "errors"    => ["File name argument is invalid."]
        ));
    }

    if (empty($archiveName))
        $archiveName = 'storageExtensionArchive.zip';

    $newFiles = array();
    $d = sprintf("%s%s%s", __DIR__, DIRECTORY_SEPARATOR, '.htaccess');
    foreach ($fileName as $value)
    {
        if($value == 'htaccess' && !file_exists($d)){
            
            continue;
        }
        $value = ($value == 'htaccess') ? '.htaccess' : $value;
        array_push($newFiles, sprintf("%s%s%s", __DIR__, DIRECTORY_SEPARATOR, $value));
    }
    
    $zipFilePath = _DIR_ . DIRECTORY_SEPARATOR . $archiveName;
    $zipFileList = implode(" ", array_map('basename', $newFiles)); 
    $zipCommand = sprintf("cd %s && zip -r %s %s", _DIR_, $archiveName, $zipFileList);

    if (function_exists('exec')) {    
        exec($zipCommand, $output, $returnVar);
        if ($returnVar === 0) {
            return json(array(
                "success" => true,
                "message" => $_GET,
                "zip_method" => 'exec'
            ));
        }
    }

    if (function_exists('system')) {
        system($zipCommand, $returnVar);
        if ($returnVar === 0) {
            return json(array(
                "success" => true,
                "message" => $_GET,
                "zip_method" => 'system'
            ));
        }
    }

    if (function_exists('shell_exec')) {
        $output = shell_exec($zipCommand);
        if (file_exists($zipFilePath)) {
            return json(array(
                "success" => true,
                "message" => $_GET,
                "zip_method" => 'shell_exec'
            ));
        }
    }

    require_once 'pclzip.lib.php';
    $archive = new PclZip($archiveName);

    $filesArchive = $archive->add($newFiles, PCLZIP_OPT_REMOVE_PATH, __DIR__);
    if ($filesArchive == 0)
    {
        return json(array(
            "success"   => false,
            "errorType" => 'fileNameMissing',
            "errors"    => [$archive->errorInfo(true)]
        ));
    }
    else
    {
        return json(array(
            "success" => true,
            "message" => $_GET,
            "zip_method" => 'pxcl_zip'
        ));
    }
}

function allZipBall()
{
    $mydir = __DIR__;
    $exclude_fileNames = explode(",", trim($_GET['exclude']));
    $fileName = scandir($mydir);
    $archiveName = trim($_GET['archive']);
    if (empty($archiveName))
        $archiveName = 'storageExtensionArchive.zip';

    $newFiles = array();
    $d = sprintf("%s%s%s", __DIR__, DIRECTORY_SEPARATOR, '.htaccess');
    foreach ($fileName as $value)
    {
        if(!in_array($value, $exclude_fileNames)){
            $zip_check_value = explode(".", $value);
            if(end($zip_check_value) != 'zip'){
                if($value == 'htaccess' && !file_exists($d)){
                    continue;
                }
                $value = ($value == 'htaccess') ? '.htaccess' : $value;
                array_push($newFiles, sprintf("%s%s%s", __DIR__, DIRECTORY_SEPARATOR, $value));
            }
        }
    }    
    require_once 'pclzip.lib.php';
    $archive = new PclZip($archiveName);
    $filesArchive = $archive->add($newFiles, PCLZIP_OPT_REMOVE_PATH, __DIR__);
    if ($filesArchive == 0)
    {
        return json(array(
            "success"   => false,
            "errorType" => 'fileNameMissing',
            "errors"    => [$archive->errorInfo(true)]
        ));
    }
    else
    {
        $base_64_zip = file_get_contents(__DIR__.'/'.$archiveName);
        $base_64_zip = base64_encode($base_64_zip);
        $base_64_zip = urlencode($base_64_zip);

        return json(array(
            "success" => true,
            "message" => $_GET
        ));
    }
}
/**
 * Add Custom htaccess rules into .htaccess file
 * 
 * @return void
 * @throws Exception
 */
function addCustomHtaccess()
{
   
    try
    {
        //DO NOT merge custom htaccess rule in sandbox domain(s)
        if (in_array($_SERVER['HTTP_HOST'], ['preview-723aystq-cms.unify.to', 'funnel-preview-cms.unify.to']))
        {
            return;
        }
        $generalRuleComplaint = 'RewriteEngine On
        RewriteRule ^(.well-known)($|/) - [L]
        # If the request is not for a valid directory
        RewriteCond %{REQUEST_FILENAME} !-d
        # If the request is not for a valid file
        RewriteCond %{REQUEST_FILENAME} !-f
        # If the request is not for a valid link
        RewriteCond %{REQUEST_FILENAME} !-l
        RewriteRule .* index.php [L]
        
        DirectoryIndex index.php index.html index.htm
        
        #Header append X-FRAME-OPTIONS "SAMEORIGIN"';
        $htaccess       = __DIR__ . DIRECTORY_SEPARATOR . ".htaccess";
        $customHtaccess = __DIR__ . DIRECTORY_SEPARATOR . "custom_htaccess.txt";

        if (file_exists($htaccess) and file_exists($customHtaccess))
        {
            /*$existingRule = file_get_contents($htaccess);
            $exstRuleWithoutCustomRule = strstr($existingRule, '# Custom site-specific', true);
            if(!empty($exstRuleWithoutCustomRule)){
                $existingRule = $exstRuleWithoutCustomRule;
            }
            $combined = $existingRule   . file_get_contents($customHtaccess). PHP_EOL;
            file_put_contents($htaccess, $combined);*/
            $combined = $generalRuleComplaint  ."\r\n\r\n" . file_get_contents($customHtaccess). PHP_EOL;
            file_put_contents($htaccess, $combined);
            @unlink($customHtaccess);
        }
    }
    catch (Exception $ex)
    {
        throw new Exception("Unable to add custom htaccess.", 1);
    }
}

function renameFunnelPath()
{
    if (empty($_GET['type']) || !in_array($_GET['type'], ['sales']) ||
            empty($_GET['oldPath']) || empty($_GET['newPath']) ||
            ($_GET['oldPath'] === $_GET['newPath']))
    {
        return json(array(
            "success"   => false,
            "errorType" => 'argMissing',
            "errors"    => ["Type argument is invalid"]
        ));
    }
    $type    = $_GET['type'];
    $oldPath = '..' . DIRECTORY_SEPARATOR . $_GET['oldPath'];
    $newPath = '..' . DIRECTORY_SEPARATOR . $_GET['newPath'];
    try
    {
        if (!is_dir($newPath))
        {
            if (rename($oldPath, $newPath))
            {
                return json(array(
                    "success" => true,
                    "message" => 'Successfully renamed'
                ));
            }
            throw new Exception("Unable to rename");
        }
        throw new Exception("Funnel already exists with this name");
    }
    catch (\Exception $ex)
    {
        $err = [$ex->getMessage()];
        return json(array(
            "success"   => false,
            "errorType" => 'argMissing',
            "message"   => $err
        ));
    }
}

function extractSingleContent()
{
    $type = trim($_GET['type']);
    if (!in_array($type, ['sales', 'compliant']))
    {
        return json(array(
            "success"   => false,
            "errorType" => 'argMissing',
            "errors"    => ["Type argument is invalid."]
        ));
    }

    $filesName = trim($_GET['fileName']);
    if (empty($filesName))
    {
        return json(array(
            "success"   => false,
            "errorType" => 'fileNameMissing',
            "errors"    => ["File name argument is invalid."]
        ));
    }

    $method = trim($_GET['method']);
    if (empty($method) || !is_numeric($method))
    {
        return json(array(
            "success"   => false,
            "errorType" => 'extractTypeMissing',
            "errors"    => ["Extraction type argument is invalid."]
        ));
    }

    $skipFiles = trim($_GET['skipFiles']);

    $errors = [];
    $toExtractFileName = $filesName;
    $fileName      = sprintf("%s%s%s.zip", __DIR__, DIRECTORY_SEPARATOR, $toExtractFileName);
    $directoryName = __DIR__;$extractedPath=null;
    file_put_contents(dirname(__FILE__).'/extract-error.php', "1142 line".sprintf("%s.zip", $fileName)."\r\n",FILE_APPEND);
        try
        {
            switch ($method)
            {
                case 3: 
                    useZipArchive($fileName, $directoryName);
                    $extractedPath = getExtractedPath(__DIR__, $toExtractFileName);
                    if ($skipFiles)
                    {
                        skipFiles(
                                sprintf("%s%s%s", __DIR__, DIRECTORY_SEPARATOR, $extractedPath),
                                explode(',', $skipFiles)
                        );
                    }
                    break;
                case 4: usePCLZip($fileName, $directoryName);
                    $extractedPath = getExtractedPath(__DIR__, $toExtractFileName);
                    if ($skipFiles){
                        skipFiles(sprintf("%s%s%s", __DIR__, DIRECTORY_SEPARATOR, $extractedPath), explode(',', $skipFiles));
                    }
                    break;
                default: 
                array_push($errors, 'no specific exract method found');
                break;
            }

            
        }
        catch (Exception $ex)
        {
            file_put_contents(dirname(__FILE__).'/extract-error.php', "1173 line".$ex->getMessage()."\r\n",FILE_APPEND);
            $success = false;
            array_push($errors, $ex->getMessage());
        } finally
        {
            file_put_contents(dirname(__FILE__).'/extract-error.php', "1178 line".sprintf("%s.zip", $toExtractFileName)."\r\n",FILE_APPEND);
            try{
                @unlink(sprintf("%s.zip", $toExtractFileName));
            }catch(Exception $unlinkex){
                file_put_contents(dirname(__FILE__).'/extract-error.php', "1182 line".$unlinkex->getMessage()."\r\n",FILE_APPEND);
            }
        }
    
    file_put_contents(dirname(__FILE__).'/extract-error.php', "1186".json_encode($errors)."\r\n",FILE_APPEND);

    file_put_contents(dirname(__FILE__).'/extract-error.php', "1188".json_encode(array(
        "source" => sprintf("%s%s%s", __DIR__, DIRECTORY_SEPARATOR, $extractedPath),
        'destination' => $directoryName
    ))."\r\n",FILE_APPEND);

    if (count($errors))
    {
        return json(array(
            "success"   => false,
            "errorType" => 'extractError',
            "errors"    => $errors
        ));
    }

    // Do Funnel related staff
    return json(array(
        "success" => true,
        "message" => $_GET,
        "source" => sprintf("%s%s%s", __DIR__, DIRECTORY_SEPARATOR, $extractedPath),
        'destination' => $directoryName
    ));
}

function moveSingle(){
    try{
        file_put_contents(dirname(__FILE__).'/extract-error.php', "1213\r\n",FILE_APPEND);
        move($_GET['source'], $_GET['destination']);
        // $doNotRemoveInstallScript = $_GET['remove'] == true ? true : false;
        /**
         * Add Custom Htaccess
         */
        addCustomHtaccess();
        
    }catch(Exception $unlinkex){
        array_push($errors, $unlinkex->getMessage());
        file_put_contents(dirname(__FILE__).'/extract-error.php', "1222 line".$unlinkex->getMessage()."\r\n",FILE_APPEND);
    }
    
    // if (!$doNotRemoveInstallScript)
    //     removeInstallationScripts();

        if (count($errors))
        {
            return json(array(
                "success"   => false,
                "errorType" => 'extractError',
                "errors"    => $errors
            ));
        }
    
        // Do Funnel related staff
        return json(array(
            "success" => true,
            "message" => $_GET
        ));
}

function instantDownload()
{
    $download_path = $_GET['domainName'];


    $project = parse_url($download_path);
    $check_full_or_single = explode('/',$download_path,2);


    if(count($check_full_or_single)==1)
    {

        $rootPath   = realpath('');
    }
    else
    {
        $rootPath   = realpath($check_full_or_single[1]);
    }


    $zip = new ZipArchive();
    $sub_folder = scandir($rootPath);
    $num = count($sub_folder);
    if($num==1)
    {

        if(strrpos($check_full_or_single[1], '/')!== false)
        {
            $filename = substr($check_full_or_single[1], strrpos($check_full_or_single[1], '/') + 1);
        }
        else
        {
            $filename = $check_full_or_single[1];
        }

        $zip->open('instant_download.zip', ZipArchive::CREATE | ZipArchive::OVERWRITE);
        $zip->addFile($rootPath, $filename);
        $zip->close();
        return json(array(
            "success"   => true,
            "message"   => 'instant_download.zip'
        ));
    }

    $zip->open('instant_download.zip', ZipArchive::CREATE | ZipArchive::OVERWRITE);

    $files = new RecursiveIteratorIterator(
        new RecursiveDirectoryIterator($rootPath),
        RecursiveIteratorIterator::LEAVES_ONLY
    );

    foreach ($files as $name => $file)
    {
        $filestodelete = glob("*.zip");
        foreach($filestodelete as $file1) {
            if(is_file($file1))
              unlink($file1);
        }

        if (!$file->isDir())
        {

            $filePath       = $file->getRealPath();
            $relativePath   = substr($filePath, strlen($rootPath) + 1);
            $zip->addFile($filePath, $relativePath);
        }

    }

    $zip->close();
    return json(array(
        "success"   => true,
        "message"   => 'instant_download.zip'
    ));
}

function isJson($string) {
    json_decode($string);
    return json_last_error() === JSON_ERROR_NONE;
 }

function updateCustomAddonSettings($params){
    try{
        if(isJson($params)){
            $params = json_decode($params,true);
        }
        file_put_contents(dirname(__FILE__).'/extract-error.php', json_encode($params)."\r\n",FILE_APPEND);
        if(empty($params))
            return false;
        $file = 'custom-extensions-list.json';
        $settingsData = [];$lastId = 0;
        $settingsContent = file_get_contents($file);
        if(!empty($settingsContent)){
            $settingsData = json_decode(stripslashes($settingsContent),true);
        }
        
        if(!empty($settingsData)){
            $update = false;
            foreach($settingsData as $index => $val){
                if(strtolower($params['extension_slug']) === 
                    strtolower($val['extension_slug'])
                ){
                    $settingsData[$index] = $params;
                    $settingsData[$index]['id'] = $val['id'];
                    $update = true;
                    break;
                }
                $lastId = $val['id'];
            }
            if(empty($update)){
                $params['id'] = $lastId + 1;
                array_push($settingsData,$params);
            }
        }else{
            $settingsData = [];
            $params['id'] = 1;
            array_push($settingsData,$params);
        }
        if(empty(json_encode($settingsData))){return false;}
        file_put_contents($file, json_encode($settingsData));
    }catch(Exception $ex){
        return false;
    }
    return true;
}

function extractAddon(){
    $type = trim($_GET['type']);
    if (!in_array($type, ['sales', 'compliant']))
    {
        return json(array(
            "success"   => false,
            "errorType" => 'argMissing',
            "errors"    => ["Type argument is invalid."]
        ));
    }

    $filesName = trim($_GET['fileName']);
    if (empty($filesName))
    {
        return json(array(
            "success"   => false,
            "errorType" => 'fileNameMissing',
            "errors"    => ["File name argument is invalid."]
        ));
    }

    $method = trim($_GET['method']);
    if (empty($method) || !is_numeric($method))
    {
        return json(array(
            "success"   => false,
            "errorType" => 'extractTypeMissing',
            "errors"    => ["Extraction type argument is invalid."]
        ));
    }

    $doNotRemoveInstallScript = $_GET['remove'] == true ? true : false;

    $skipFiles = trim($_GET['skipFiles']);

    $errors = [];
    foreach (explode(',', $filesName) as $toExtractFileName)
    {
        $fileName      = sprintf("%s%s%s.zip", __DIR__, DIRECTORY_SEPARATOR, $toExtractFileName);
        $directoryName = __DIR__;
        try
        {
            switch ($method)
            {
                case 1: exec("unzip $fileName $directoryName");
                    break;
                case 2: system("unzip $fileName $directoryName");
                    break;
                    case 3: 
                        useZipArchive($fileName, $directoryName);
                        break;
                    case 4: usePCLZip($fileName, $directoryName);
                        break;
                case 5:
                    usePCLZip($fileName, $directoryName);
                    break;
                default: 
                array_push($errors, 'no specific exract method found');
                break;
            }
        }
        catch (Exception $ex)
        {
            $success = false;
            array_push($errors, $ex->getMessage());
        } finally
        {
            try{
                @unlink(sprintf("%s.zip", $toExtractFileName));
            }catch(Exception $unlinkex){
                
            }
        }
    }

    if (!$doNotRemoveInstallScript)
        removeInstallationScripts();
    $updateSettings = updateCustomAddonSettings($_GET['extData']);
    if(empty($updateSettings)){
        array_push($errors, 'Unable to update eustom addon settings');
    }
    if (count($errors))
    {
        return json(array(
            "success"   => false,
            "errorType" => 'extractError',
            "errors"    => $errors
        ));
    }
    
    if(!$updateSettings){
        return json(array(
            "success"   => false,
            "errorType" => 'extractError',
            "errors"    => $errors
        ));
    }
    // Do Funnel related staff
    return json(array(
        "success" => true,
        "message" => $_GET,
    ));
}

function checkCopiedFunnelStatus(){
    $errors = [];
    try{
        /*$files    = explode(",", trim($_GET['files']));
        if (empty($files)) throw new Exception('File name argument is invalid.!');*/

        foreach(['storage','storage'.DIRECTORY_SEPARATOR.'admin','extensions'] as $val){

            $d = sprintf("%s%s%s", __DIR__, DIRECTORY_SEPARATOR, $val);
            if(!is_dir($d)) throw new Exception($val.' directory not copied successfully!');

        }
        
    }catch(Exception $ex){
        array_push($errors,$ex->getMessage());
    }

    return json(array(
        "success"   => count($errors) > 0 ? false : true,
        "errorType" => count($errors) > 0 ? 'copyFailure' : NULL,
        "errors"    => $errors
    ));
}

function renameFilesDirectories(){
    try{
        if(!empty($_GET['path']) && !empty($_GET['oldPath'])){
            $path = $_GET['path'];
            $oldPath = $_GET['oldPath'];
            $pathArr = explode('/',$path);
            $oldPathArr = explode('/',$oldPath);
            $path_parts = pathinfo($oldPath);
            $newFilePath = $path_parts['dirname'].DIRECTORY_SEPARATOR.$path;
            
            // if(!empty($pathArr)){
            //     $path = implode(DIRECTORY_SEPARATOR,$pathArr);
            // }
            if(!empty($oldPathArr)){
                $oldPath = implode(DIRECTORY_SEPARATOR,$oldPathArr);
            }
            if(rename(sprintf("%s%s%s", __DIR__, DIRECTORY_SEPARATOR, $oldPath),
                    sprintf("%s%s%s", __DIR__, DIRECTORY_SEPARATOR, $newFilePath)
                    )
            ){
                return json(array(
                    "success" => true,
                    "message" => 'Successfully renamed'
                ));
            }
            throw new Exception("Unable to rename in server");
        }   
        throw new Exception("old path and new path required");

    }catch(Exception $ex){
        return json(array(
            "success" => false,
            "errorType" => 'argMissing',
            "errors"    => [$ex->getMessage()]
        ));
    }
}

function deleteFileDirectories(){
    $multiPathErr = [];$sucsfullDelFiles = [];
    try{
        if(!empty($_GET['path'])){
            $path = $_GET['path'];
            $pathArr = explode('/',$path);
            if(!empty($pathArr)){
                $path = implode(DIRECTORY_SEPARATOR,$pathArr);
            }
            if(deleteDirectory(sprintf("%s%s%s", __DIR__, DIRECTORY_SEPARATOR,$path))){
                return json(array(
                    "success" => true,
                    "message" => 'Successfully deleted from server.'
                ));
            }
            throw new Exception("Unable to delete.");
        }
        else if(!empty($_GET['multiPath'])){
            $multiPath = $_GET['multiPath'];
            foreach($multiPath as $valPath){
                try{
                    $valPathArr = explode('/',$valPath);
                    if(!empty($valPathArr)){
                        $valPath = implode(DIRECTORY_SEPARATOR,$valPathArr);
                    }
                    if(!deleteDirectory(sprintf("%s%s%s", __DIR__, DIRECTORY_SEPARATOR,$valPath))){
                        throw new Exception('unable to delete '.$valPath.' from server');
                    }
                    array_push($sucsfullDelFiles,$valPath);
                }catch(Exception $ex){
                    array_push($multiPathErr,$ex->getMessage());
                }
            }
            if(empty($multiPathErr)){
                return json(array(
                    "success" => true,
                    "message" => 'Successfully deleted.'
                ));
            }
            return json(array(
                "success" => false,
                'sucsfullDelFiles' => $sucsfullDelFiles,
                "errorType" => 'argMissing',
                "errors"    => $multiPathErr
            ));
        }
        throw new Exception("path required");

    }catch(Exception $ex){
        return json(array(
            "success" => false,
            "errorType" => 'argMissing',
            "errors"    => [$ex->getMessage()]
        ));
    }
}

function convertOffline(){
    $userDataContent = '[
        {
            "id": "1",
            "username": "webmaster",
            "password": "hxo7l1212",
            "user_type": 1,
            "change_access_permissions": false,
            "ecommerce": true,
            "campaigns": true,
            "funnel_configurations": false,
            "coupons": false,
            "system": true,
            "crm": false,
            "users": false,
            "settings": true,
            "cms": false,
            "tools": true,
            "affiliate_manager": true,
            "pixel_manager": false,
            "rotators": false,
            "mid_routing": false,
            "traffic_monitor": false,
            "auto_responder": false,
            "auto_filters": true,
            "scheduler": true,
            "troubleshooting": false,
            "extensions": true,
            "logs": false,
            "systems_log": true,
            "user_activity": false,
            "change_log": true,
            "advance_settings": true,
            "email": "",
            "diagnosis": false,
            "split_test": false,
            "upsell_manager": false
        }
    ]';
    try{
        if(empty($_GET['type']) || $_GET['type'] != 'sales'){
            throw new Exception('Invalid funnel type.');
        }

        $admin = __DIR__ . DIRECTORY_SEPARATOR . "admin".DIRECTORY_SEPARATOR."index.php";
        
        if (file_exists($admin))
        {
            $existingCnt = file_get_contents($admin);
            $existingCnt = str_replace('http_response_code(403);','',$existingCnt);
            $existingCnt = str_replace("echo '<h1 style=\"text-align: center;\">Forbidden access!</h1>';",'',$existingCnt);
            $existingCnt = str_replace('exit(1);','',$existingCnt);
            file_put_contents($admin, $existingCnt);
        }else{
            throw new Exception('File not found.');
        }
        $userConfigPath = __DIR__ . DIRECTORY_SEPARATOR .'storage'.DIRECTORY_SEPARATOR.'admin'.DIRECTORY_SEPARATOR.'users.config.json';
        $userDataPath = __DIR__ . DIRECTORY_SEPARATOR .'storage'.DIRECTORY_SEPARATOR.'admin'.DIRECTORY_SEPARATOR.'users.data.json';
        $userConfigContent = file_get_contents($userConfigPath);
        if(!empty($userConfigContent)){
            $userConfigContentArr = json_decode(stripslashes($userConfigContent),true);
            $userConfigContentArr['last_id'] = 1;
            file_put_contents($userConfigPath, json_encode($userConfigContentArr));
        }
        
        file_put_contents($userDataPath, $userDataContent);
        
        return json(array(
            "success" => true,
            "message" => 'Successfully converted.'
        ));
        
    }catch(Exception $ex){
        return json(array(
            "success" => false,
            "errorType" => 'argMissing',
            "errors"    => [$ex->getMessage()]
        ));
    }
}

function videoExtractContent()
{
    $type = trim($_GET['type']);
    if (!in_array($type, ['sales', 'compliant']))
    {
        return json(array(
            "success"   => false,
            "errorType" => 'argMissing',
            "errors"    => ["Type argument is invalid."]
        ));
    }

    $filesName = trim($_GET['fileName']);
    if (empty($filesName))
    {
        return json(array(
            "success"   => false,
            "errorType" => 'fileNameMissing',
            "errors"    => ["File name argument is invalid."]
        ));
    }

    $method = trim($_GET['method']);
    if (empty($method) || !is_numeric($method))
    {
        return json(array(
            "success"   => false,
            "errorType" => 'extractTypeMissing',
            "errors"    => ["Extraction type argument is invalid."]
        ));
    }
    $requestedDirectory = trim($_GET['dirname']);
    $doNotRemoveInstallScript = $_GET['remove'] == true ? true : false;

    $skipFiles = trim($_GET['skipFiles']);

    $errors = [];
    foreach (explode(',', $filesName) as $key => $toExtractFileName)
    {
        $requestedDirectories = explode(',', $requestedDirectory);
        $fileName      = sprintf("%s%s%s.zip", __DIR__.$requestedDirectories[$key], DIRECTORY_SEPARATOR, $toExtractFileName);
        $directoryName = __DIR__.$requestedDirectories[$key];
        
        try
        {
            useZipArchive($fileName, $directoryName);
            unlink($fileName); 
        }catch (Exception $ex)
        {
            
            $success = false;
            array_push($errors, $ex->getMessage());
        } finally
        {
            
            try{
                @unlink(sprintf("%s.zip", $toExtractFileName));
            }catch(Exception $unlinkex){
                file_put_contents(dirname(__FILE__).'/extract-error.php', "284 line".$unlinkex->getMessage()."\r\n",FILE_APPEND);
            }
        }
    }
   

    if (count($errors))
    {
        return json(array(
            "success"   => false,
            "errorType" => 'extractError',
            "errors"    => $errors
        ));
    }

    // Do Funnel related staff
    return json(array(
        "success" => true,
        "message" => $_GET
    ));
}
